"""
module to warn user that function are deprecated

# === Examples of use ===

@deprecated
def some_old_function(x,y):
    return x + y

class SomeClass:
    @deprecated
    def some_old_method(self, x,y):
        return x + y

"""

import warnings
import functools


def deprecated(func):
    """This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used."""

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        """
        new_func
        """
        warnings.warn_explicit(
            "Call to deprecated function %(funcname)s." % {"funcname": func.__name__},
            category=DeprecationWarning,
            filename=func.__code__.co_filename,
            lineno=func.__code__.co_firstlineno + 1,
        )

        print("\n\n*********** LAUETOOLS warning *************        ")
        print("Call to deprecated function %(funcname)s" % {"funcname": func.__name__})
        print(
            "in module %(filename)s at linenumber %(linenumber)d."
            % {
                "filename": func.__code__.co_filename,
                "linenumber": func.__code__.co_firstlineno + 1,
            }
        )
        print("*" * 43 + "\n\n")
        return func(*args, **kwargs)

    return new_func


@deprecated
def some_old_function(x, y):
    """
    example
    """
    return x + y



# -*- coding: utf-8 -*-
r"""
IOimagefile module is made for reading data contained in binary image file
fully or partially.

More tools can be found in LaueTools package at sourceforge.net and gitlab.esrf.fr
March 2020
"""
__author__ = "Jean-Sebastien Micha, CRG-IF BM32 @ ESRF"

# built-in modules
import sys
import os
import copy
import struct

# third party modules
try:
    import warnings
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore",category=FutureWarning)
        import fabio

    FABIO_EXISTS = True
except ImportError:
    print("Missing fabio module. Please install it if you need open some tiff images "
            "from the sCMOS camera")
    FABIO_EXISTS = False

try:
    from libtiff import TIFF, libtiff_ctypes

    libtiff_ctypes.suppress_warnings()
    LIBTIFF_EXISTS = True
except (ImportError, ValueError):
    print("Missing library libtiff, Please install: pylibtiff if you need open some tiff images")
    LIBTIFF_EXISTS = False

try:
    from PIL import Image

    PIL_EXISTS = True
except ImportError:
    print("Missing python module called PIL. Please install it if you need open some tiff "
            "images from vhr camera")
    PIL_EXISTS = False
import numpy as np

try:
    from lauetools import dict_LaueTools as DictLT
    from lauetools import imageprocessing as ImProc
except:
    import lauetoolsnn.lauetools.dict_LaueTools as DictLT
    import lauetoolsnn.lauetools.imageprocessing as ImProc
     
listfile = os.listdir(os.curdir)

def stringint(k, n):
    r""" returns string of integer k with n zeros padding
    (by placing zeros before to have n characters)

    :param k: integer to convert
    :param n: nb of digits for zero padding

    :return: string of length n containing integer k

    Example: 1 -> '0001'
    15 -> '0015'
    """
    #     strint = str(k)
    #     res = '0' * (n - len(strint)) + strint

    encodingdigits = "{" + ":0{}".format(int(n)) + "}"
    res = encodingdigits % k

    return res

# --- ---------------- read images functions
def setfilename(imagefilename, imageindex, nbdigits=4, CCDLabel=None, verbose=0):
    r"""
    reconstruct filename string from imagefilename and update filename index with imageindex

    :param imagefilename: filename string (full path or not)
    :param imageindex: index in filename
    :type imageindex: string

    :return filename: input filename with index replaced by input imageindex
    :rtype: string
    """

    #     print "imagefilename",imagefilename
    if imagefilename.endswith("mccd"):
        lenext = 5 #length of extension including '.'
        imagefilename = imagefilename[: -(lenext + nbdigits)] + "{:04d}.mccd".format(imageindex)

    elif CCDLabel in ("sCMOS", "sCMOS_fliplr"):
        #default file extension for sCMOS camera
        ext = "tif"
        lenext = 4  #length of extension including '.'

        if imagefilename.endswith("tiff"):
            ext = "tiff"
            lenext = 5
        # zero padded index for filename
        if nbdigits is not None:
            if imagefilename.endswith(ext):
                #imagefilename = imagefilename[: - (lenext + nbdigits)] + "{:04d}.{}".format(imageindex, ext)

                #imagefilename = imagefilename[: - (lenext + nbdigits)].zfill(nbdigits) + "%s" % ext

                prepart = str(imagefilename).rsplit('.',1)[0]
                prefix, digitpart = str(prepart).rsplit('_', 1)

                current_nbdigits = len(digitpart)
                imagefilename = prefix + "_" + str(imageindex).zfill(current_nbdigits) + "." + ext
                
            elif imagefilename.endswith(ext+".gz"):
                imagefilename = imagefilename[: -(lenext+3 + nbdigits)] + "{:04d}.{}.gz".format(imageindex, ext)
        # no zero padded index for filename
        else:
            if imagefilename.endswith(ext):
                prefix, _ = imagefilename.split(".")
                prefix0 = prefix.rsplit("_")[0]
                if imageindex > 9999:
                    imagefilename = prefix0 + "_{}.{}".format(imageindex, ext)
                else:
                    imagefilename = prefix0 + "_{:04d}.{}".format(imageindex, ext)

    elif CCDLabel in ("EIGER_4Mstack",):
        # only stackimageindex is changed not imagefilename
        pass

    #     print "imagefilename archetype", imagefilename
    elif imagefilename.endswith("mar.tiff"):

        imagefilename = imagefilename[: -(9 + nbdigits)] + "{:04d}_mar.tiff".format(imageindex)

    elif imagefilename.endswith("mar.tif"):
        # requires two underscores with number in between
        # map1_105_mar.tif
        sname = imagefilename.split("_")
        if len(sname) >= 4:  # pathfilename contain more than 1 underscore

            sname2 = sname[-2] + "_" + sname[-1]
            prefix = imagefilename.rstrip(sname2)
        else:
            prefix = imagefilename.rsplit("_")[0]
        imagefilename = prefix + "_{}_mar.tif".format(imageindex)

    # special case for image id15 frelon corrected form distorsion
    elif imagefilename.endswith((".tif", ".edf")):
        if CCDLabel in ("ImageStar_raw", ):
            prefixwihtindex = imagefilename[:-4]
            prefixwithindex_list = list(prefixwihtindex)
            indexnodigit = 0
            while len(prefixwithindex_list) > 0:
                lastelem = prefixwithindex_list.pop(-1)
                if verbose>0: print("lastelem", lastelem)
                if not lastelem.isdigit():
                    break
                indexnodigit += 1
            prefix = prefixwihtindex[:-(indexnodigit)]
            if verbose > 0: print("prefix", prefix)
            imagefilename = prefix + "{}.tif".format(imageindex)

        else:
            suffix = imagefilename[-4:]
            prefix = imagefilename[: -(4 + nbdigits)]

            imagefilename = prefix + "{:04d}{}".format(imageindex, suffix)

    elif imagefilename.endswith("mccd"):

        imagefilename = imagefilename[: -(5 + nbdigits)] + "{:04d}.mccd".format(imageindex)

    elif imagefilename.endswith("edf"):

        imagefilename = imagefilename[: -(4 + nbdigits)] + "{:04d}.edf".format(imageindex)

    elif imagefilename.endswith("unstacked"):

        imagefilename = imagefilename[: -(10 + nbdigits)] + "{:04d}.unstacked".format(imageindex)

    #     print "set filename to:", imagefilename
    return imagefilename


def getIndex_fromfilename(imagefilename, nbdigits=4, CCDLabel=None, stackimageindex=-1, verbose=0):
    r"""
    get integer index from imagefilename string

    :param imagefilename: filename string (full path or not)

    :return: file index
    """
    if CCDLabel in ("sCMOS", "sCMOS_fliplr"):
        #default file extension for sCMOS camera
        ext = "tif"
        lenext = 4 #length of extension including '.'

        if imagefilename.endswith("tiff"):
            ext = "tiff"
            lenext = 5
        if verbose > 0: print('imagefilename', imagefilename)
        if nbdigits is not None:
            if imagefilename.endswith(ext):
                #imageindex = int(imagefilename[-(lenext + nbdigits): - (lenext)])
                prepart = str(imagefilename).rsplit('.',1)[0]
                digitpart = str(prepart).rsplit('_',1)[-1]
                print("prepart", prepart)
                print("digitpart",digitpart)
                imageindex = int(digitpart)
            elif imagefilename.endswith(ext+".gz"):
                imageindex = int(imagefilename[-(lenext+3 + nbdigits) : -(lenext+3)])
        else:
            if imagefilename.endswith(ext):
                prefix, _ = imagefilename.rsplit(".")
                imageindex = int(prefix.rsplit("_")[1])

    # for stacked images we return the position of image data in the stack as imagefileindex
    elif CCDLabel in ("EIGER_4Mstack",):
        imageindex = stackimageindex

    elif imagefilename.endswith("mar.tiff"):
        imageindex = int(imagefilename[-(9 + nbdigits) : -9])
    elif imagefilename.endswith("tiff"):
        imageindex = int(imagefilename[-(5 + nbdigits) : -5])
    elif imagefilename.endswith("mar.tif"):
        imageindex = int(imagefilename.split("_")[-2])
    elif imagefilename.endswith(".tif"):
        # TODO: treat the case of HN56.tif  without underscore (see setfilename)
        imageindex = int(imagefilename.split("_")[-1][:-4])
    elif imagefilename.endswith("mccd"):
        imageindex = int(imagefilename[-(5 + nbdigits) : -5])
    elif imagefilename.endswith("edf"):
        imageindex = int(imagefilename[-(4 + nbdigits) : -4])
    elif imagefilename.endswith("unstacked"):
        imageindex = int(imagefilename[-(10 + nbdigits) : -10])

    return imageindex


def getfilename(dirname, imfileprefix, imfilesuffix=None, numim=None,
                nbdigits_filename=4):
    """
    to get the global file name (name+path) for given components of the name
    put %4d instead of stringint
    """

    fileim = imfileprefix + stringint(numim, nbdigits_filename) + imfilesuffix
    filename = os.path.join(dirname, fileim)

    return filename

def getwildcardstring(CCDlabel):
    r"""  return smart wildcard to open binary CCD image file with priority of CCD type of CCDlabel

    :param CCDlabel: string label defining the CCD type
    :type CCDlabel: str
    :return: string from concatenated strings to be used in wxpython open file dialog box
    :rtype: str

    .. see also::
        - :func:`getIndex_fromfilename`

        - LaueToolsGUI.AskUserfilename

        - wx.FileDialog

    .. example::
        >>> from readmccd import getwildcardstring
        >>> getwildcardstring('MARCCD165')
        'MARCCD, ROPER(*.mccd)|*mccd|mar tif(*.tif)|*_mar.tiff|tiff(*.tiff)|*tiff|Princeton(*.spe)|*spe|Frelon(*.edf)|*edf|tif(*.tif)|*tif|All files(*)|*'
    """
    ALL_EXTENSIONS = ["mccd", "_mar.tiff", "tiff", "spe", "edf", "tif", "h5", ""]
    INFO_EXTENSIONS = ["MARCCD, ROPER(*.mccd)", "mar tif(*.tif)", "tiff(*.tiff)",
                    "Princeton(*.spe)", "Frelon(*.edf)", "tif(*.tif)", "hdf5(*.h5)",
                    "All files(*)"]

    extensions = copy.copy(ALL_EXTENSIONS)
    infos = copy.copy(INFO_EXTENSIONS)
    chosen_extension = DictLT.dict_CCD[CCDlabel][7]

    if chosen_extension in ALL_EXTENSIONS:
        index = ALL_EXTENSIONS.index(chosen_extension)
        ce = extensions.pop(index)
        extensions.insert(0, ce)

        inf = infos.pop(index)
        infos.insert(0, inf)

    wcd = ""
    for inf, ext in zip(infos, extensions):
        wcd += "{}|*{}|".format(inf, ext)

    wildcard_extensions = wcd[:-1]

    return wildcard_extensions


def getpixelValue(filename, x, y, ccdtypegeometry="edf"):
    r"""return pixel value at x,y

    .. warning::
        Very old function. To be checked. Use better readpixelvalue in plotdip.py

    :param filename: path to image file
    :type filename: str
    :param x: x pixel value
    :type x: int
    :param y: y pixel value
    :type y: int
    :param ccdtypegeometry: CCD label, defaults to "edf"
    :type ccdtypegeometry: str, optional
    :return: pixel intensity
    :rtype: int
    """
    if ccdtypegeometry == "edf":
        # frelon camera as mounted on BM32 Oct2012
        y = 2047 - y
    if ccdtypegeometry in ("mccd",):
        pass

    f = open(filename, "rb")
    f.seek(1024 + 2 * (2048 * y + x))
    val = struct.unpack("H", f.read(2))
    f.close()
    return val[0]


def readheader(filename, offset=4096, CCDLabel="MARCCD165"):
    r"""
    return header in a raw format

    default offset for marccd image
    """
    if CCDLabel.startswith("sCMOS"):
        filesize = os.path.getsize(filename)
        framedim = DictLT.dict_CCD[CCDLabel][0]
        offset = filesize - np.prod(framedim) * 2
    elif CCDLabel.startswith("MARCCD165"):
        offset = 4096
    with open(filename, "rb") as f:
        myheader = f.read(offset)
        #myheader.replace("\x00", " ")

    return myheader


def read_header_marccd(filename):
    r"""
    return string of parameters found in header in marccd image file .mccd

    - print allsentences  displays the header
    - use allsentences.split('\n') to get a list
    """
    f = open(filename, "rb")
    f.seek(2048)
    posbyte = 0
    allsentences = ""
    for _ in list(range(32)):
        tt = f.read(32)
        s1 = tt.strip("\x00")
        if s1 != "":
            allsentences += s1 + "\n"
        # print posbyte, s1
        posbyte += 32
    tt = f.read(1024)
    s1 = tt.strip("\x00")
    if s1 != "":
        allsentences += s1 + "\n"

    f.close()
    return allsentences


def read_header_marccd2(filename):
    r"""
    return string of parameters comments and exposure time
    found in header in marccd image file .mccd

    - print allsentences  displays the header
    - use allsentences.split('\n') to get a list
    """
    f = open(filename, "rb")
    f.seek(3072)
    tt = f.read(512)
    # from beamline designed input
    dataset_comments = str(tt).strip("\x00")

    f.seek(1024 + 2 * 256 + 128 + 12)
    s = struct.Struct("I I I")
    unpacked_data = s.unpack(f.read(3 * 4))
    #     print 'Unpacked Values:', unpacked_data
    #integration_time, expo_time, readout_time = unpacked_data
    _, expo_time, _ = unpacked_data

    f.close()
    return dataset_comments, expo_time


def read_header_scmos(filename, verbose=0):
    r"""
    return string of parameters comments and exposure time
    found in header in scmis image file .tif

    - print allsentences  displays the header
    - use allsentences.split('\n') to get a list
    """
    if not PIL_EXISTS:
        return {}

    img = Image.open(filename)


    # img.tag.keys()
    # tag[270]   = (u'0 (thf=-50.4850 mon=24923 exposure=0.400)',)

    dictpar = {}
    strcom = img.tag[270][0]
    if verbose>0: print('read_header_scmos', strcom, type(strcom))

    si = strcom.index("(")
    fi = strcom.index(")")
    listpar = strcom[si + 1 : fi].split()
    #     print "listpar",listpar
    for elem in listpar:
        if "=" in elem:
            key, val = elem.split("=")
            dictpar[key] = float(val)

    return dictpar


def read_motorsposition_fromheader(filename, CCDLabel="MARCCD165"):
    r""" return xyzpositions, expo_time from image file header
    available for "MARCCD165", "sCMOS", "sCMOS_fliplr"
    """
    if CCDLabel in ("MARCCD165",):
        dataset_comments, expo_time = read_header_marccd2(filename)
        xyz = dataset_comments.split(" ")[:3]
        xyzpositions = []
        for pos in xyz:
            xyzpositions.append(float(pos))
    elif CCDLabel in ("sCMOS", "sCMOS_fliplr"):
        dictpar = {}
        import re

        img = Image.open(filename)
        ff = img.tag[315]
        aa = re.findall(r"([^:]+)=([^:]+)(?:,|$)", ff[0])
        listeq = aa[0][0].split(" ")
        for elem in listeq:
            if "Xech" in elem or "Yech" in elem or "Zech" in elem:
                dictpar[elem.lstrip("(")] = float(elem.rstrip(")"))

        headerdict = read_header_scmos(filename)
        if headerdict:
            expo_time = headerdict["exposure"]
        xyzpositions = []
        for motorname in ("Xech", "Yech", "Zech"):
            xyzpositions.append(dictpar[motorname])

    return xyzpositions, expo_time


def readoneimage_full(filename, frametype="mccd", dirname=None):
    r"""
    too SLOW!
    reads 1 entire image (marCCD format)
    :return: PILimage, image object of PIL module (16 bits integer) and
    arrayofdata: 2D array of intensity
    #TODO: manage framedim like readoneimage() just below"""

    if frametype == "mccd":
        shapeCCD = (2048, 2048)
    elif frametype == "martiff":
        # opposite of fit2d displays in loading data
        shapeCCD = (2591, 2751)
    elif frametype == "spe":
        # opposite of fit2d displays in loading data
        shapeCCD = (2048, 2048)

    if dirname is None:
        dirname = os.curdir

    pilimage = Image.open(os.path.join(dirname, filename))
    ravdata = np.array(pilimage.getdata())

    return pilimage, np.reshape(ravdata, shapeCCD)


def readCCDimage(filename, CCDLabel="MARCCD165", dirname=None, stackimageindex=-1, verbose=0):
    r"""Read raw data binary image file.

    Read raw data binary image file and return pixel intensity 2D array such as
    to fit the data (2theta, chi) scattering angles representation convention.

    :param filename: path to image file (fullpath if ` dirname` =None)
    :type filename: str
    :param CCDLabel: label, defaults to "MARCCD165"
    :type CCDLabel: str, optional
    :param dirname: folder path, defaults to None
    :type dirname: str, optional
    :param stackimageindex: index of images bunch, defaults to -1
    :type stackimageindex: int, optional
    :param verbose: 0 or 1, defaults to 0
    :type verbose: int, optional
    :raises ValueError: if data format and CCD parameters from label are not compatible
    :return:
        - dataimage, 2D array image data pixel intensity properly oriented
        - framedim, iterable of 2 integers shape of dataimage
        - fliprot : string, key for CCD frame transform to orient image
    :rtype: tuple of 3 elements
    """
    (framedim, _, _, fliprot, offsetheader, formatdata, _, _) = DictLT.dict_CCD[CCDLabel]

    USE_RAW_METHOD = False

    if verbose > 1:
        print("CCDLabel in readCCDimage", CCDLabel)
        print('FABIO_EXISTS',FABIO_EXISTS)
        print('LIBTIFF_EXISTS',LIBTIFF_EXISTS)
        print('PIL_EXISTS',PIL_EXISTS)
    #    if extension != extension:
    #        print "warning : file extension does not match CCD type set in Set CCD File Parameters"
    if FABIO_EXISTS:
        if CCDLabel in ('MARCCD165', "EDF", "EIGER_4M", "EIGER_1M",
                        "sCMOS", "sCMOS_fliplr", "sCMOS_fliplr_16M", "sCMOS_16M",
                        "Rayonix MX170-HS", 'psl_weiwei', 'ImageStar_dia_2021'):

            if verbose > 1:
                print('----> Using fabio ... to open %s\n'%filename)
            # warning import Image  # for well read of header only

            if dirname is not None:
                img = fabio.open(os.path.join(dirname, filename))
            else:
                img = fabio.open(filename)

            dataimage = img.data
            framedim = dataimage.shape

            # pythonic way to change immutable tuple...
            initframedim = list(DictLT.dict_CCD[CCDLabel][0])
            initframedim[0] = framedim[0]
            initframedim[1] = framedim[1]
            initframedim = tuple(initframedim)
        else:
            USE_RAW_METHOD = True

    elif CCDLabel in ("EIGER_4Mstack", ):

        import tables as Tab

        if dirname is not None:
            pathtofile = os.path.join(dirname, filename)
        else:
            pathtofile = filename

        # TODO check to correct version transition
        if Tab.__version__ >= "3.4.2":
            hdf5file = Tab.open_file(pathtofile)
        else:
            hdf5file = Tab.openFile(pathtofile)

        if verbose > 0: print("opening hdf5 stacked data table")
        alldata = hdf5file.root.entry.data.data
        if verbose > 0: print("alldata.shape", alldata.shape)

        dataimage = alldata[stackimageindex]
        framedim = dataimage.shape

    elif LIBTIFF_EXISTS:
        if verbose > 1:
            print("----> Using libtiff...")
        if CCDLabel in ("sCMOS", "MARCCD165", "TIFF Format", "FRELONID15_corrected", "VHR_PSI",
                            "VHR_DLS", "MARCCD225", "Andrea", "pnCCD_Tuba", "sCMOS_16M"):

            #         print "tiff format", CCDLabel
            #             print "dirname, filename", dirname, filename
            if dirname is not None:
                tifimage = TIFF.open(os.path.join(dirname, filename), mode="r")
            else:
                tifimage = TIFF.open(filename, mode="r")

            dataimage = tifimage.read_image()
            framedim = (int(tifimage.GetField("ImageLength")),
                        int(tifimage.GetField("ImageWidth")))
            if tifimage.IsByteSwapped():
                dataimage = dataimage.byteswap()
        else:
            USE_RAW_METHOD = True

    elif PIL_EXISTS:
        if verbose > 1:
            print("using PIL's module Image")
        if CCDLabel in ("sCMOS", "MARCCD165", "sCMOS_16M"):
            if verbose > 1: print('PIL is too slow. Better install libtiff or fabio. Meanwhile ...')
            USE_RAW_METHOD = True
        elif CCDLabel in ("VHR_PSI", "VHR_DLS", "MARCCD225", "Andrea", "pnCCD_Tuba"):
            # data are compressed!

            if dirname is not None:
                fullpath = os.path.join(dirname, filename)
            else:
                fullpath = filename

            im = Image.open(fullpath, "r")
            dataimage = np.array(im.getdata()).reshape(framedim)

    # RAW method knowing or deducing offsetheader and dataformat
    if USE_RAW_METHOD:
        print("----> !!! not using libtiff, nor fabio, nor PIL!!!  ")
        if CCDLabel in ("MARCCD165",):
            print("for MARCCD not using libtiff, raw method ...")
            # offsetheader may change ...
            #filesize=getfilesize(dirname, filename)
            offsetheader = 4096 #filesize - 2048*2048 * 2
        # offset header varying
        elif CCDLabel.startswith("ImageStar_raw"):
            filesize=getfilesize(dirname, filename)
            bytes_per_pixels = 2
            if CCDLabel.endswith("32bits"):
                bytes_per_pixels = 4

            nbpixels = 1500
            if CCDLabel in ("ImageStar_1528x1528",):
                nbpixels = 1528
            offsetheader = filesize - nbpixels * nbpixels * bytes_per_pixels

        elif CCDLabel == "ImageStar_dia_2021":
            filesize=getfilesize(dirname, filename)
            bytes_per_pixels = 2
            nbpixels = 3056

            offsetheader = filesize - nbpixels * nbpixels * bytes_per_pixels
        elif CCDLabel in ("sCMOS",):
            if verbose > 0: print("for sCMOS not using libtiff, raw method ...")
            # offsetheader may change ...
            filesize = getfilesize(dirname, filename)
            offsetheader = filesize - 2016*2018 * 2

        dataimage = readoneimage(filename,
                                framedim=framedim,
                                dirname=dirname,
                                offset=offsetheader,
                                formatdata=formatdata)

        print("dataimage", dataimage)

        if CCDLabel in ("FRELONID15_corrected",):

            dataimage = dataimage.byteswap()

        if CCDLabel in ("EIGER_4Munstacked",):
            dataimage = np.ma.masked_where(dataimage > 4000000, dataimage)
            if verbose > 0:
                print("framedim", framedim)
                print("offsetheader", offsetheader)
                print("formatdata", formatdata)
                print("dataimage", dataimage)

    if verbose:
        print("CCDLabel: ", CCDLabel)
        print("nb of pixels", np.shape(dataimage))

    # need to reshape data from 1D to 2D
    try:
        if len(dataimage.shape) == 1:
            if verbose:
                print("nb elements", len(dataimage))
                print("framedim", framedim)
                print("framedim nb of elements", framedim[0] * framedim[1])
            dataimage = np.reshape(dataimage, framedim)
    except ValueError:
        raise ValueError(
            "Selected CCD type :{} may be wrong (or nb of pixels, dimensions...)".format(
                CCDLabel))

    # some array transformations if needed depending on the CCD mounting
    if fliprot == "spe":
        dataimage = np.rot90(dataimage, k=1)

    elif fliprot == "VHR_Feb13":
        #            self.dataimage_ROI = np.rot90(self.dataimage_ROI, k=3)
        dataimage = np.fliplr(dataimage)

    elif fliprot == "sCMOS_fliplr":
        dataimage = np.fliplr(dataimage)

    elif fliprot == "vhr":  # july 2012 close to diamond monochromator crystal
        dataimage = np.rot90(dataimage, k=3)
        dataimage = np.fliplr(dataimage)

    elif fliprot == "vhrdiamond":  # july 2012 close to diamond monochromator crystal
        dataimage = np.rot90(dataimage, k=3)
        dataimage = np.fliplr(dataimage)

    elif fliprot == "Imagestar_dia_2021":  # starting Feb july 2021 close to diamond monochromator crystal
        dataimage = np.rot90(dataimage, k=1)
        dataimage = np.fliplr(dataimage)

    elif fliprot == "frelon2":
        dataimage = np.flipud(dataimage)

    return dataimage, framedim, fliprot

def getfilesize(dirname,filename):
    if dirname is not None:
        filesize = os.path.getsize(os.path.join(dirname, filename))
    else:
        filesize = os.path.getsize(filename)
    return filesize

def readoneimage(filename, framedim=(2048, 2048), dirname=None, offset=4096, formatdata="uint16"):
    r""" returns a 1d array of integers from a binary image file (full data)

    :param filename: image file name (full path if dirname=0)
    :type filename: str
    :param framedim: detector dimensions, defaults to (2048, 2048)
    :type framedim: tuple of 2 integers, optional
    :param dirname: folder path, defaults to None
    :type dirname: str, optional
    :param offset: file header in byte (octet), defaults to 4096
    :type offset: int, optional
    :param formatdata: numpy format of raw binary image pixel value, defaults to "uint16"
    :type formatdata: str, optional
    :return: dataimage : image data pixel intensity
    :rtype: 1D array
    """

    nb_elem = framedim[0] * framedim[1]

    if dirname is None:
        dirname = os.curdir

    f = open(os.path.join(dirname, filename), "rb")
    f.seek(offset)

    # d=scipy.io.fread(f,2048*2048,np.oldnumeric.Int16)
    # d = scipy.io.fread(f,nb_elem,np.oldnumeric.UInt16)
    d = np.fromfile(f, dtype=formatdata, count=nb_elem)
    f.close()
    return d


def readoneimage_band(filename,
                        framedim=(2048, 2048),
                        dirname=None,
                        offset=4096,
                        line_startindex=0,
                        line_finalindex=2047,
                        formatdata="uint16"):
    r"""
    returns a 1d array of integers from a binary image file. Data located in band according shape of data (framedim)

    :param filename: string
               path to image file (fullpath if `dirname`=None)
    :param offset: integer
             nb of file header bytes
    :param framedim: iterable of 2 integers
               shape of expected 2D data
    :param formatdata: string
                 key for numpy dtype to decode binary file

    :return: dataimage, 1D array, image data pixel intensity
    """
    if dirname is None:
        dirname = os.curdir

    if formatdata in ("uint16",):
        nbBytesPerElement = 2
    if formatdata in ("uint32",):
        nbBytesPerElement = 4

    nbElems = (line_finalindex - line_startindex + 1) * framedim[1]

    f = open(os.path.join(dirname, filename), "rb")
    f.seek(offset + line_startindex * framedim[1] * nbBytesPerElement)

    # d=scipy.io.fread(f,2048*2048,np.oldnumeric.Int16)
    # d = scipy.io.fread(f,nb_elem,np.oldnumeric.UInt16)
    #     print "line_finalindex-line_startindex",line_finalindex-line_startindex
    #     print "line_startindex*framedim[1]*nbBytesPerElement",line_startindex*framedim[1]*nbBytesPerElement
    #     print "line_startindex*framedim[1]",line_startindex*framedim[1]
    #     print "nbElems",nbElems

    band = np.fromfile(f, dtype=formatdata, count=nbElems)
    f.close()
    return band


def readoneimage_crop_fast(filename, dirname=None, CCDLabel="MARCCD165",
                                                    firstElemIndex=0, lastElemIndex=2047, verbose=0):
    r""" Returns a 2d array of integers from a binary image file. Data are taken only from a rectangle

    with respect to firstElemIndex and lastElemIndex.

    :param filename: string, path to image file (fullpath if ` dirname`=None)
    :param offset: integer, nb of file header bytes
    :param framedim: iterable of 2 integers, shape of expected 2D data
    :param formatdata: string, key for numpy dtype to decode binary file

    :return: dataimage : 1D array image data pixel intensity
    """
    (framedim, _, _, fliprot, offsetheader, formatdata, _, _) = DictLT.dict_CCD[CCDLabel]
    if verbose > 0:
        print("framedim read from DictLT.dict_CCD in readoneimage_crop_fast()", framedim)
        print("formatdata", formatdata)
        print("offsetheader", offsetheader)

    if dirname is None:
        dirname = os.curdir

    # if formatdata in ("uint16",):
    #     nbBytesPerElement = 2
    # if formatdata in ("uint32",):
    #     nbBytesPerElement = 4

    dataimage2D = np.zeros(framedim)

    # colFirstElemIndex = firstElemIndex % framedim[1]
    lineFirstElemIndex = firstElemIndex // framedim[1]
    # colLastElemIndex = lastElemIndex % framedim[1]
    lineLastElemIndex = lastElemIndex // framedim[1]

    band1D = readoneimage_band(filename, framedim=framedim, dirname=dirname, offset=offsetheader,
                                                            line_startindex=lineFirstElemIndex,
                                                            line_finalindex=lineLastElemIndex,
                                                            formatdata=formatdata)

    #     print "band1D.shape",band1D.shape
    #     print "(lineLastElemIndex-lineFirstElemIndex,framedim[1])",(lineLastElemIndex-lineFirstElemIndex,framedim[1])
    #     print "((lineLastElemIndex-lineFirstElemIndex)*framedim[1])",((lineLastElemIndex-lineFirstElemIndex)*framedim[1])

    band2D = band1D.reshape((lineLastElemIndex - lineFirstElemIndex + 1, framedim[1]))

    dataimage2D[lineFirstElemIndex : lineLastElemIndex + 1, :] = band2D

    return dataimage2D, framedim, fliprot


def readrectangle_in_image(filename, pixx, pixy, halfboxx, halfboxy, dirname=None,
                                                                            CCDLabel="MARCCD165",
                                                                            verbose=0):
    r"""
    returns a 2d array of integers from a binary image file. Data are taken only from a rectangle
    centered on pixx, pixy

    :return: dataimage : 2D array, image data pixel intensity
    """
    (framedim, _, _, fliprot, offsetheader, formatdata, _, _) = DictLT.dict_CCD[CCDLabel]

    if verbose > 0:
        print("framedim read from DictLT.dict_CCD in readrectangle_in_image()", framedim)
        print("formatdata", formatdata)
        print("offsetheader", offsetheader)
    # recompute headersize
    if dirname is not None:
        fullpathfilename = os.path.join(dirname, filename)
    else:
        dirname = os.curdir
        fullpathfilename = filename

    if formatdata in ("uint16",):
        nbBytesPerElement = 2
    if formatdata in ("uint32",):
        nbBytesPerElement = 4

    if verbose > 0:
        print("fullpathfilename", fullpathfilename)
    try:
        filesize = os.path.getsize(fullpathfilename)
    except OSError:
        print("missing file {}\n".format(fullpathfilename))
        return None

    # uint16
    offsetheader = filesize - (framedim[0] * framedim[1]) * nbBytesPerElement

    if verbose > 0:
        print("calculated offset of header from file size...", offsetheader)

    x = int(pixx)
    y = int(pixy)

    if fliprot in ("sCMOS_fliplr",):
        x = framedim[1] - x

    boxx = int(halfboxx)
    boxy = int(halfboxy)

    xpixmin = x - boxx
    xpixmax = x + boxx

    ypixmin = y - boxy
    ypixmax = y + boxy

    #     ymin= (y-boxy)*framedim[1]
    #     ymax= (y+boxy)*framedim[1]

    lineFirstElemIndex = ypixmin
    lineLastElemIndex = ypixmax

    if verbose > 0:
        print("lineFirstElemIndex", lineFirstElemIndex)
        print("lineLastElemIndex", lineLastElemIndex)

    band = readoneimage_band(fullpathfilename,
                                framedim=framedim,
                                dirname=None,
                                offset=offsetheader,
                                line_startindex=lineFirstElemIndex,
                                line_finalindex=lineLastElemIndex,
                                formatdata=formatdata)

    nblines = lineLastElemIndex - lineFirstElemIndex + 1

    band2D = np.reshape(band, (nblines, framedim[1]))

    rectangle2D = band2D[:, xpixmin : xpixmax + 1]

    if verbose > 0:
        print("band2D.shape", band2D.shape)
        print("rectangle2D.shape", rectangle2D.shape)

    return rectangle2D


def readoneimage_crop(filename, center, halfboxsize, CCDLabel="PRINCETON", dirname=None):
    r"""
    return a cropped array of data read in an image file

    :param filename: string, path to image file (fullpath if ` dirname`=None)
    :param center: iterable of 2 integers, (x,y) pixel coordinates
    :param halfboxsize: integer or iterable of 2 integers, ROI half size in both directions

    :return: dataimage : 1D array, image data pixel intensity

    .. todo:: useless?
    """
    if dirname is None:
        dirname = os.curdir
    else:
        filename = os.path.join(dirname, filename)

    if isinstance(halfboxsize, int):
        boxsizex, boxsizey = halfboxsize, halfboxsize
    elif len(halfboxsize) == 2:
        boxsizex, boxsizey = halfboxsize

    xpic, ypic = center

    dataimage, framedim, _ = readCCDimage(filename, CCDLabel=CCDLabel, dirname=None)

    x1 = np.maximum(0, xpic - boxsizex)
    x2 = np.minimum(framedim[1], xpic + boxsizex + 1)  # framedim[0]
    y1 = np.maximum(0, ypic - boxsizey)
    y2 = np.minimum(framedim[0], ypic + boxsizey + 1)  # framedim[1]

    return None, dataimage[y1:y2, x1:x2]


def readoneimage_manycrops(filename, centers, boxsize, stackimageindex=-1, CCDLabel="MARCCD165",
                                                                        addImax=False,
                                                                        use_data_corrected=None,
                                                                        verbose=0):
    r"""
    reads 1 image and extract many regions
    centered on center_pixel with xyboxsize dimensions in pixel unit

    :param filename: string,fullpath to image file
    :param centers: list or array of [int,int] centers (x,y) pixel coordinates
    :param use_data_corrected: enter data instead of reading data from file
                         must be a tuple of 3 elements:
                         fulldata, framedim, fliprot
                         where fulldata is a numpy.ndarray
                         as output by :func:`readCCDimage`
    :param boxsize: iterable 2 elements or integer
              boxsizes [in x, in y] direction or integer to set a square ROI

    :return: Data, list of 2D array pixel intensity or Data and Imax
    """
    # use alternate data  (for instance for data from filename without background)
    if use_data_corrected is not None:
        if isinstance(use_data_corrected, tuple):
            if len(use_data_corrected) == 3:
                fulldata, framedim, _ = use_data_corrected
    # use data by reading file
    else:
        fulldata, framedim, _ = readCCDimage(filename, stackimageindex=stackimageindex,
                                                            CCDLabel=CCDLabel,
                                                            dirname=None)

    if isinstance(boxsize, int):
        boxsizex, boxsizey = boxsize, boxsize
    elif len(boxsize) == 2:
        boxsizex, boxsizey = boxsize

    # xpic, ypic = np.array(centers).T

    #    x1 = np.array(np.maximum(0, xpic - boxsizex), dtype=np.int)
    #    x2 = np.array(np.minimum(framedim[0], xpic + boxsizex), dtype=np.int)
    #    y1 = np.array(np.maximum(0, ypic - boxsizey), dtype=np.int)
    #    y2 = np.array(np.minimum(framedim[1], ypic + boxsizey), dtype=np.int)

    Data = []

    Imax = []
        
    if verbose > 0:
        print("framedim in readoneimage_manycrops", framedim)
    framedim = framedim[1], framedim[0]

    for center in centers:
        i1, i2, j1, j2 = ImProc.getindices2cropArray(center, (boxsizex, boxsizey), framedim)
        #        print "i1, i2, j1, j2-----", i1, i2, j1, j2
        cropdata = fulldata[i1:i2, j1:j2]
        #         # print "cropdata.shape", cropdata.shape
        #         print "i2-i1",i2-i1
        #         print "boxsizey*2+1",boxsizey*2+1
        #         print "j2-j1",j2-j1
        #         print "boxsizex*2+1",boxsizex*2+1

        # for spot near border, replace by zeros array
        if i2 - i1 != boxsizey * 2 or j2 - j1 != boxsizex * 2:
            cropdata = np.zeros((boxsizey * 2 + 1, boxsizex * 2 + 1))

        Data.append(cropdata)
        if addImax:
            Imax.append(np.amax(cropdata))

    #        print "max in cropped data", np.amax(cropdata)
    if addImax:
        return Data, Imax
    else:
        return Data

def writeimage(outputname, _header, data, dataformat=np.uint16, verbose=0):
    r"""
    from data 1d array of integers
    with header coming from a f.open('imagefile'); f.read(headersize);f.close()
    .. warning:: header contain dimensions for subsequent data. Check before the compatibility of
    data with header infos(nb of byte per pixel and array dimensions
    """
    newfile = open(outputname, "wb")
    newfile.write(_header)
    data = np.array(data, dtype=dataformat)
    data.tofile(newfile)
    newfile.close()
    if verbose > 0: print("image written in ", outputname)


def write_rawbinary(outputname, data, dataformat=np.uint16, verbose=0):
    r"""
    write a binary file without header of a 2D array

    used ?
    """
    newfile = open(outputname, "wb")
    data = np.array(data, dtype=dataformat)
    data.tofile(newfile)

    newfile.close()
    if verbose > 0: print("image written in ", outputname)


def SumImages(prefixname, suffixname, ind_start, ind_end, dirname=None,
                                                            plot=0,
                                                            output_filename=None,
                                                            CCDLabel=None,
                                                            nbdigits=0):
    r"""
    sum images and write image with 32 bits per pixel format (4 bytes)

    used?
    """
    #     prefixname = 'HN08_'
    #     suffixname = '.tif'
    #     CCDLabel = 'ImageStar_raw'
    #     dirname = '/home/micha/LaueProjects/Vita'

    output_filename = "mean_{}_{:04d}_{}{}".format(prefixname, ind_start, ind_end, suffixname)

    filename = "{}{:04d}{}".format(prefixname, ind_start, suffixname)

    data, shape, _ = readCCDimage(filename, CCDLabel=CCDLabel, dirname=dirname)

    if CCDLabel == "ImageStar_raw":
        # Add addition of 32 bits image => replace 2 by 4  nb of bytes per pixel
        filesize = os.path.getsize(os.path.join(dirname, filename))
        offsetheader = filesize - 1500 * 1500 * 2
    if CCDLabel == "EIGER_1M":
        filesize = os.path.getsize(os.path.join(dirname, filename))
        offsetheader = filesize - 1065 * 1030 * 4
    else:
        # Add addition of 32 bits image => replace 2 by 4  nb of bytes per pixel
        filesize = os.path.getsize(os.path.join(dirname, filename))
        offsetheader = filesize - 2048 * 2048 * 2
    # print "shape = ", np.shape(datastart)
    datasum = np.zeros(shape, dtype=np.uint32)

    #     nb_images = ind_end - ind_start + 1

    indexscanlist = list(range(ind_start, ind_end + 1, 1))
    for k in indexscanlist:
        # print k
        filename = "{}{:04d}{}".format(prefixname, k, suffixname)
        # print filename1
        data, shape, _ = readCCDimage(filename, CCDLabel=CCDLabel, dirname=dirname)
        # print max(data1), np.argmax(data1)
        datasum = data + datasum

    if output_filename:
        outputfilename = output_filename
        header = readheader(os.path.join(dirname, filename), offset=offsetheader)
        writeimage(os.path.join(dirname, outputfilename), header, datasum, dataformat=np.uint32)

        print("Added images with prefix {} from {} to {} written in {}".format(
                prefixname, ind_start, ind_end, outputfilename))
    if plot:
        print("later")

    return datasum


def Add_Images2(prefixname,
                ind_start,
                ind_end,
                plot=0,
                writefilename=None,
                CCDLabel="MARCCD165",
                average=True):
    """
    in dev
    """
    suffixname = "." + DictLT.dict_CCD[CCDLabel][-1]

    filename = prefixname + stringint(ind_start, 4) + suffixname
    datastart, _, _ = readCCDimage(filename, CCDLabel)

    # print "shape = ", np.shape(datastart)
    datastart = np.array(datastart, dtype=float)

    indexscanlist = list(range(ind_start + 1, ind_end + 1, 1))
    for k in indexscanlist:
        # print k
        filename1 = prefixname + stringint(k, 4) + suffixname
        # print filename1
        data1, _, _ = readCCDimage(filename1, CCDLabel)
        # print max(data1), np.argmax(data1)
        datastart = np.array(data1, dtype=float) + datastart
    # datastart= datastart/((len(indexscanlist)+1)*1.)
    # print "final"
    # print max(datastart), np.argmax(datastart)
    if average:
        datastart = datastart / (float(len(indexscanlist)) + 1.0)
    # print max(datastart), np.argmax(datastart)
    datastart = np.array(datastart, dtype=np.uint16)
    # print max(datastart), np.argmax(datastart)
    if writefilename:
        outputfilename = writefilename
        header = readheader(filename1, CCDLabel=CCDLabel)
        writeimage(outputfilename, header, datastart)
        print("written in ", outputfilename)
    if plot:
        print("later")

    return datastart


def Add_Images(prefixname, ind_start, ind_end, plot=0, writefilename=None):
    r"""
    Add continuous sequence of images

    .. note::
        Add_Images2 exists

    :param prefixname: string, prefix common part of name of files

    :param ind_start: int, starting image index

    :param ind_end: int, final image index

    :param writefilename: string, new image filename where to write datastart (with last image file header read)

    :return:  datastart, array accumulation of 2D data from each image
    """
    suffixname = ".mccd"

    datastart = readoneimage(prefixname + stringint(ind_start, 4) + suffixname)
    # print "shape = ", np.shape(datastart)
    datastart = np.array(datastart, dtype=float)

    indexscanlist = list(range(ind_start + 1, ind_end + 1, 1))
    for k in indexscanlist:
        # print k
        filename1 = prefixname + stringint(k, 4) + suffixname
        # print filename1
        data1 = readoneimage(filename1)
        # print max(data1), np.argmax(data1)
        datastart = np.array(data1, dtype=float) + datastart
    # datastart= datastart/((len(indexscanlist)+1)*1.)
    # print "final"
    # print max(datastart), np.argmax(datastart)
    datastart = datastart / (float(len(indexscanlist)) + 1.0)
    # print max(datastart), np.argmax(datastart)
    datastart = np.array(datastart, dtype=np.uint16)
    # print max(datastart), np.argmax(datastart)
    if writefilename:
        outputfilename = writefilename
        header = readheader(filename1)
        writeimage(outputfilename, header, datastart)
        print("written in ", outputfilename)
    if plot:
        print("later")

    return datastart

def get_imagesize(framedim, nbbits_per_pixel, headersize_bytes):
    r"""
    return size of image in byte (= 1 octet = 8 bits)
    """
    return (framedim[0] * framedim[1] * nbbits_per_pixel + headersize_bytes * 8) // 8



# -*- coding: utf-8 -*-
"""
Created on Sun Sep 12 20:09:37 2021

@author: PURUSHOT

Post processing example scripts for results with LaueToolsNN
"""

import _pickle as cPickle
import matplotlib.pyplot as plt
import numpy as np
import os
from mpl_toolkits.axes_grid1 import make_axes_locatable
import lauetoolsnn.lauetools.CrystalParameters as CP
import lauetoolsnn.lauetools.dict_LaueTools as dictLT
import scipy
from scipy.spatial.transform import Rotation as R

def rot_mat_to_euler(rot_mat): 
    r = R.from_matrix(rot_mat)
    return r.as_euler('zxz')* 180/np.pi

results_folder = r"path to results directory"
    
with open(results_folder+ "//results.pickle", "rb") as input_file:
    best_match, mat_global, rotation_matrix1, strain_matrix, strain_matrixs,\
        col, colx, coly, match_rate, files_treated,\
            lim_x, lim_y, spots_len, iR_pix, fR_pix, material_, \
                material1_, lattice_, lattice1_,\
                symmetry, symmetry1, crystal, crystal1 = cPickle.load(input_file)

## results directory
save_directory_ = os.getcwd() + "\\" + results_folder.split("\\")[-1]
if not os.path.exists(save_directory_):
    os.makedirs(save_directory_)
    
match_tol = 25
fR_tol = 2
matnumber = 1
bins = 100
rangeval = len(match_rate)
material_id = [material_, material1_]
matid = 0

#%%establish mu sigma range for all dataset
mu_sd = []
count = 0
for index in range(rangeval):
    ### index for nans
    nan_index11 = np.where(match_rate[index][0] < match_tol)[0]
    nan_index1 = np.where(fR_pix[index][0] > fR_tol)[0]
    mat_id_index = np.where(mat_global[index][0] != matid+1)[0]
    nan_index = np.hstack((mat_id_index,nan_index1,nan_index11))
    nan_index = np.unique(nan_index)
    
    if count == 0:
        spots_len_plot = np.copy(spots_len[index][0])
        mr_plot = np.copy(match_rate[index][0])
        iR_pix_plot = np.copy(iR_pix[index][0])
        fR_pix_plot = np.copy(fR_pix[index][0])
        strain_matrix_plot = np.copy(strain_matrix[index][0])
        e11c = np.copy(strain_matrix_plot[:,0,0])#.reshape((lim_x, lim_y))
        e22c = np.copy(strain_matrix_plot[:,1,1])#.reshape((lim_x, lim_y))
        e33c = np.copy(strain_matrix_plot[:,2,2])#.reshape((lim_x, lim_y))
        e12c = np.copy(strain_matrix_plot[:,0,1])#.reshape((lim_x, lim_y))
        e13c = np.copy(strain_matrix_plot[:,0,2])#.reshape((lim_x, lim_y))
        e23c = np.copy(strain_matrix_plot[:,1,2])#.reshape((lim_x, lim_y))
        spots_len_plot[nan_index] = np.nan 
        mr_plot[nan_index] = np.nan 
        iR_pix_plot[nan_index] = np.nan 
        fR_pix_plot[nan_index] = np.nan 
        e11c[nan_index] = np.nan 
        e22c[nan_index] = np.nan 
        e33c[nan_index] = np.nan 
        e12c[nan_index] = np.nan 
        e13c[nan_index] = np.nan 
        e23c[nan_index] = np.nan 
        count = 1
        
    else:
        temp = np.copy(spots_len[index][0])
        temp[nan_index] = np.nan
        spots_len_plot = np.vstack((spots_len_plot,temp))
        
        temp = np.copy(match_rate[index][0])
        temp[nan_index] = np.nan
        mr_plot = np.vstack((mr_plot,temp))
        
        temp = np.copy(iR_pix[index][0])
        temp[nan_index] = np.nan
        iR_pix_plot = np.vstack((iR_pix_plot,temp))

        temp = np.copy(fR_pix[index][0])
        temp[nan_index] = np.nan
        fR_pix_plot = np.vstack((fR_pix_plot,temp))
        
        strain_matrix_plot = np.copy(strain_matrix[index][0])
        temp = np.copy(strain_matrix_plot[:,0,0])
        temp[nan_index] = np.nan
        e11c = np.vstack((e11c,temp))
        temp = np.copy(strain_matrix_plot[:,1,1])
        temp[nan_index] = np.nan
        e22c = np.vstack((e22c,temp))
        temp = np.copy(strain_matrix_plot[:,2,2])
        temp[nan_index] = np.nan
        e33c = np.vstack((e33c,temp))
        temp = np.copy(strain_matrix_plot[:,0,1])
        temp[nan_index] = np.nan
        e12c = np.vstack((e12c,temp))
        temp = np.copy(strain_matrix_plot[:,0,2])
        temp[nan_index] = np.nan
        e13c = np.vstack((e13c,temp))
        temp = np.copy(strain_matrix_plot[:,1,2])
        temp[nan_index] = np.nan
        e23c = np.vstack((e23c,temp))

spots_len_plot = spots_len_plot.flatten()
mr_plot = mr_plot.flatten()
iR_pix_plot = iR_pix_plot.flatten()
fR_pix_plot = fR_pix_plot.flatten() 
e11c = e11c.flatten()
e22c = e22c.flatten()
e33c = e33c.flatten()
e12c = e12c.flatten()
e13c = e13c.flatten()
e23c = e23c.flatten()

spots_len_plot = spots_len_plot[~np.isnan(spots_len_plot)]
mr_plot = mr_plot[~np.isnan(mr_plot)]
iR_pix_plot = iR_pix_plot[~np.isnan(iR_pix_plot)]
fR_pix_plot = fR_pix_plot[~np.isnan(fR_pix_plot)]
e11c = e11c[~np.isnan(e11c)]
e22c = e22c[~np.isnan(e22c)]
e33c = e33c[~np.isnan(e33c)]
e12c = e12c[~np.isnan(e12c)]
e13c = e13c[~np.isnan(e13c)]
e23c = e23c[~np.isnan(e23c)]

logdata = e11c
estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
mu_sd.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))

logdata = e22c
estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
mu_sd.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))

logdata = e33c
estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
mu_sd.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))

logdata = e12c
estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
mu_sd.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))

logdata = e13c
estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
mu_sd.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))

logdata = e23c
estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
mu_sd.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))


#%% Compute lattice params from strain
for index in range(rangeval):

    constantlength = "a"
    
    a,b,c,alp,bet,gam = [],[],[],[],[],[]
    #TODO all images and not only one
    for irot in range(len(rotation_matrix1[index][0])):
        if (match_rate[index][0][irot] < match_tol) or \
            fR_pix[index][0][irot] > fR_tol:
            continue
        
        lattice_parameter_direct_strain = CP.computeLatticeParameters_from_UB(rotation_matrix1[index][0][irot,:,:], 
                                                                              material_, 
                                                                              constantlength, 
                                                                              dictmaterials=dictLT.dict_Materials)
        a.append(lattice_parameter_direct_strain[0])
        b.append(lattice_parameter_direct_strain[1])
        c.append(lattice_parameter_direct_strain[2])
        alp.append(lattice_parameter_direct_strain[3])
        bet.append(lattice_parameter_direct_strain[4])
        gam.append(lattice_parameter_direct_strain[5])
    
    title = "Refined unit cell"+" "+material_id[0]+ " "+str(index)
    fig = plt.figure()
    axs = fig.subplots(2, 3)
    axs[0, 0].set_title(r"a", loc='center', fontsize=8)
    logdata = np.array(a)
    logdata = logdata[~np.isnan(logdata)]
    rangemin, rangemax = np.min(logdata)-0.01, np.max(logdata)+0.01
    axs[0, 0].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
    axs[0, 0].set_ylabel('Frequency', fontsize=8)
    axs[0, 0].tick_params(axis='both', which='major', labelsize=8)
    axs[0, 0].tick_params(axis='both', which='minor', labelsize=8)

    axs[0, 1].set_title(r"b", loc='center', fontsize=8)
    logdata = np.array(b)
    logdata = logdata[~np.isnan(logdata)]
    rangemin, rangemax = np.min(logdata)-0.01, np.max(logdata)+0.01
    axs[0, 1].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
    axs[0, 1].set_ylabel('Frequency', fontsize=8)
    axs[0, 1].tick_params(axis='both', which='major', labelsize=8)
    axs[0, 1].tick_params(axis='both', which='minor', labelsize=8)
    
    axs[0, 2].set_title(r"c", loc='center', fontsize=8)
    logdata = np.array(c)
    logdata = logdata[~np.isnan(logdata)]
    rangemin, rangemax = np.min(logdata)-0.01, np.max(logdata)+0.01
    axs[0, 2].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
    axs[0, 2].set_ylabel('Frequency', fontsize=8)
    axs[0, 2].tick_params(axis='both', which='major', labelsize=8)
    axs[0, 2].tick_params(axis='both', which='minor', labelsize=8)
    
    axs[1, 0].set_title(r"$\alpha$", loc='center', fontsize=8)
    logdata = np.array(alp)
    logdata = logdata[~np.isnan(logdata)]
    rangemin, rangemax = np.min(logdata)-0.01, np.max(logdata)+0.01
    axs[1, 0].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
    axs[1, 0].set_ylabel('Frequency', fontsize=8)
    axs[1, 0].tick_params(axis='both', which='major', labelsize=8)
    axs[1, 0].tick_params(axis='both', which='minor', labelsize=8)
    
    axs[1, 1].set_title(r"$\beta$", loc='center', fontsize=8)
    logdata = np.array(bet)
    logdata = logdata[~np.isnan(logdata)]
    rangemin, rangemax = np.min(logdata)-0.01, np.max(logdata)+0.01
    axs[1, 1].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
    axs[1, 1].set_ylabel('Frequency', fontsize=8)
    axs[1, 1].tick_params(axis='both', which='major', labelsize=8)
    axs[1, 1].tick_params(axis='both', which='minor', labelsize=8)
    
    axs[1, 2].set_title(r"$\gamma$", loc='center', fontsize=8)
    logdata = np.array(gam)
    logdata = logdata[~np.isnan(logdata)]
    rangemin, rangemax = np.min(logdata)-0.01, np.max(logdata)+0.01
    axs[1, 2].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
    axs[1, 2].set_ylabel('Frequency', fontsize=8)
    axs[1, 2].tick_params(axis='both', which='major', labelsize=8)
    axs[1, 2].tick_params(axis='both', which='minor', labelsize=8)
    

    plt.tight_layout()
    plt.savefig(save_directory_+"\\"+title+'.png', format='png', dpi=1000) 
    plt.close(fig)

    #%% Range min max for strain plots
    rangemin = -0.3
    rangemax = 0.3
    
    #%% Plot histograms
    mu_sd1 = []
    ### index for nans
    nan_index11 = np.where(match_rate[index][0] < match_tol)[0]
    nan_index1 = np.where(fR_pix[index][0] > fR_tol)[0]
    mat_id_index = np.where(mat_global[index][0] != matid+1)[0]
    nan_index = np.hstack((mat_id_index,nan_index1,nan_index11))
    nan_index = np.unique(nan_index)
    
    spots_len_plot = np.copy(spots_len[index][0])
    mr_plot = np.copy(match_rate[index][0])
    iR_pix_plot = np.copy(iR_pix[index][0])
    fR_pix_plot = np.copy(fR_pix[index][0])
    strain_matrix_plot = np.copy(strain_matrix[index][0])
    e11c = np.copy(strain_matrix_plot[:,0,0])#.reshape((lim_x, lim_y))
    e22c = np.copy(strain_matrix_plot[:,1,1])#.reshape((lim_x, lim_y))
    e33c = np.copy(strain_matrix_plot[:,2,2])#.reshape((lim_x, lim_y))
    e12c = np.copy(strain_matrix_plot[:,0,1])#.reshape((lim_x, lim_y))
    e13c = np.copy(strain_matrix_plot[:,0,2])#.reshape((lim_x, lim_y))
    e23c = np.copy(strain_matrix_plot[:,1,2])#.reshape((lim_x, lim_y))
    strain_matrixs_plot = np.copy(strain_matrixs[index][0])
    e11s = np.copy(strain_matrixs_plot[:,0,0])#.reshape((lim_x, lim_y))
    e22s = np.copy(strain_matrixs_plot[:,1,1])#.reshape((lim_x, lim_y))
    e33s = np.copy(strain_matrixs_plot[:,2,2])#.reshape((lim_x, lim_y))
    e12s = np.copy(strain_matrixs_plot[:,0,1])#.reshape((lim_x, lim_y))
    e13s = np.copy(strain_matrixs_plot[:,0,2])#.reshape((lim_x, lim_y))
    e23s = np.copy(strain_matrixs_plot[:,1,2])#.reshape((lim_x, lim_y))
    spots_len_plot[nan_index] = np.nan 
    mr_plot[nan_index] = np.nan 
    iR_pix_plot[nan_index] = np.nan 
    fR_pix_plot[nan_index] = np.nan 
    e11c[nan_index] = np.nan 
    e22c[nan_index] = np.nan 
    e33c[nan_index] = np.nan 
    e12c[nan_index] = np.nan 
    e13c[nan_index] = np.nan 
    e23c[nan_index] = np.nan 
    e11s[nan_index] = np.nan 
    e22s[nan_index] = np.nan 
    e33s[nan_index] = np.nan 
    e12s[nan_index] = np.nan 
    e13s[nan_index] = np.nan 
    e23s[nan_index] = np.nan 
        
    spots_len_plot = spots_len_plot.flatten()
    mr_plot = mr_plot.flatten()
    iR_pix_plot = iR_pix_plot.flatten()
    fR_pix_plot = fR_pix_plot.flatten() 
    e11c = e11c.flatten()
    e22c = e22c.flatten()
    e33c = e33c.flatten()
    e12c = e12c.flatten()
    e13c = e13c.flatten()
    e23c = e23c.flatten()
    e11s = e11s.flatten()
    e22s = e22s.flatten()
    e33s = e33s.flatten()
    e12s = e12s.flatten()
    e13s = e13s.flatten()
    e23s = e23s.flatten()
    
    spots_len_plot = spots_len_plot[~np.isnan(spots_len_plot)]
    mr_plot = mr_plot[~np.isnan(mr_plot)]
    iR_pix_plot = iR_pix_plot[~np.isnan(iR_pix_plot)]
    fR_pix_plot = fR_pix_plot[~np.isnan(fR_pix_plot)]
    e11c = e11c[~np.isnan(e11c)]
    e22c = e22c[~np.isnan(e22c)]
    e33c = e33c[~np.isnan(e33c)]
    e12c = e12c[~np.isnan(e12c)]
    e13c = e13c[~np.isnan(e13c)]
    e23c = e23c[~np.isnan(e23c)]
    e11s = e11s[~np.isnan(e11s)]
    e22s = e22s[~np.isnan(e22s)]
    e33s = e33s[~np.isnan(e33s)]
    e12s = e12s[~np.isnan(e12s)]
    e13s = e13s[~np.isnan(e13s)]
    e23s = e23s[~np.isnan(e23s)]
    
    try:
        title = "Number of spots and matching rate"
        fig = plt.figure()
        axs = fig.subplots(1, 2)
        axs[0].set_title("Number of spots", loc='center', fontsize=8)
        axs[0].hist(spots_len_plot, bins=bins)
        axs[0].set_ylabel('Frequency', fontsize=8)
        axs[0].tick_params(axis='both', which='major', labelsize=8)
        axs[0].tick_params(axis='both', which='minor', labelsize=8)
        axs[1].set_title("Matching rate", loc='center', fontsize=8)
        axs[1].hist(mr_plot, bins=bins)
        axs[1].set_ylabel('Frequency', fontsize=8)
        axs[1].tick_params(axis='both', which='major', labelsize=8)
        axs[1].tick_params(axis='both', which='minor', labelsize=8)
        plt.tight_layout()
        plt.savefig(save_directory_+"\\"+title+"_"+material_id[matid]+"_"+str(index)+'.png', format='png', dpi=1000) 
        plt.close(fig)
    except:
        pass

    try:
        title = "strain Crystal reference"+" "+material_id[matid]
        fig = plt.figure()
        axs = fig.subplots(2, 3)
        axs[0, 0].set_title(r"$\epsilon_{11}$ (%)", loc='center', fontsize=8)
        logdata = e11c #np.log(e11c)
        logdata = logdata[~np.isnan(logdata)]
        estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
        mu_sd1.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))
        axs[0, 0].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
        axs[0, 0].set_ylabel('Frequency', fontsize=8)
        axs[0, 0].tick_params(axis='both', which='major', labelsize=8)
        axs[0, 0].tick_params(axis='both', which='minor', labelsize=8)
        
        axs[0, 1].set_title(r"$\epsilon_{22}$ (%)", loc='center', fontsize=8)
        logdata = e22c #np.log(e22c)
        logdata = logdata[~np.isnan(logdata)]
        estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
        mu_sd1.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))
        axs[0, 1].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
        axs[0, 1].set_ylabel('Frequency', fontsize=8)
        axs[0, 1].tick_params(axis='both', which='major', labelsize=8)
        axs[0, 1].tick_params(axis='both', which='minor', labelsize=8)
        
        axs[0, 2].set_title(r"$\epsilon_{33}$ (%)", loc='center', fontsize=8)
        logdata = e33c #np.log(e33c)
        logdata = logdata[~np.isnan(logdata)]
        estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
        mu_sd1.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))
        axs[0, 2].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
        axs[0, 2].set_ylabel('Frequency', fontsize=8)
        axs[0, 2].tick_params(axis='both', which='major', labelsize=8)
        axs[0, 2].tick_params(axis='both', which='minor', labelsize=8)
        
        axs[1, 0].set_title(r"$\epsilon_{12}$ (%)", loc='center', fontsize=8)
        logdata = e12c#np.log(e12c)
        logdata = logdata[~np.isnan(logdata)]
        estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
        mu_sd1.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))
        axs[1, 0].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
        axs[1, 0].set_ylabel('Frequency', fontsize=8)
        axs[1, 0].tick_params(axis='both', which='major', labelsize=8)
        axs[1, 0].tick_params(axis='both', which='minor', labelsize=8)
        
        axs[1, 1].set_title(r"$\epsilon_{13}$ (%)", loc='center', fontsize=8)
        logdata = e13c#np.log(e13c)
        logdata = logdata[~np.isnan(logdata)]
        estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
        mu_sd1.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))
        axs[1, 1].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
        axs[1, 1].set_ylabel('Frequency', fontsize=8)
        axs[1, 1].tick_params(axis='both', which='major', labelsize=8)
        axs[1, 1].tick_params(axis='both', which='minor', labelsize=8)
        
        axs[1, 2].set_title(r"$\epsilon_{23}$ (%)", loc='center', fontsize=8)
        logdata = e23c#np.log(e23c)
        logdata = logdata[~np.isnan(logdata)]
        estimated_mu, estimated_sigma = scipy.stats.norm.fit(logdata)
        mu_sd1.append((estimated_mu-estimated_sigma, estimated_mu+estimated_sigma))
        axs[1, 2].hist(logdata, bins=bins, density=True, alpha=0.8, range=(rangemin, rangemax))
        axs[1, 2].set_ylabel('Frequency', fontsize=8)
        axs[1, 2].tick_params(axis='both', which='major', labelsize=8)
        axs[1, 2].tick_params(axis='both', which='minor', labelsize=8)
        
        plt.tight_layout()
        plt.savefig(save_directory_+"\\"+title+"_"+str(index)+'.png', format='png', dpi=1000) 
        plt.close(fig)
    except:
        print("error")
        pass
    
    #%%    Plot strain data    

    nan_index11 = np.where(match_rate[index][0] < match_tol)[0]
    nan_index1 = np.where(fR_pix[index][0] > fR_tol)[0]
    mat_id_index = np.where(mat_global[index][0] != matid+1)[0]
    nan_index = np.hstack((mat_id_index,nan_index1,nan_index11))
    nan_index = np.unique(nan_index)

    strain_matrix_plot = np.copy(strain_matrix[index][0])
    strain_matrix_plot[nan_index,:,:] = np.nan             

    fig = plt.figure(figsize=(11.69,8.27), dpi=100)
    bottom, top = 0.1, 0.9
    left, right = 0.1, 0.8
    fig.subplots_adjust(top=top, bottom=bottom, left=left, right=right, hspace=0.15, wspace=0.25)
    
    try:
        vmin, vmax = mu_sd[matid*6]
    except:
        print("error")
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs = fig.subplots(2, 3)
    axs[0, 0].set_title(r"$\epsilon_{11}$ (%)", loc='center', fontsize=8)
    im=axs[0, 0].imshow(strain_matrix_plot[:,0,0].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    axs[0, 0].set_xticks([])
    axs[0, 0].set_yticks([])
    divider = make_axes_locatable(axs[0,0])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd[matid*6+1]
    except:
        print("error")
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[0, 1].set_title(r"$\epsilon_{22}$ (%)", loc='center', fontsize=8)
    im=axs[0, 1].imshow(strain_matrix_plot[:,1,1].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    divider = make_axes_locatable(axs[0,1])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd[matid*6+2]
    except:
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[0, 2].set_title(r"$\epsilon_{33}$ (%)", loc='center', fontsize=8)
    im=axs[0, 2].imshow(strain_matrix_plot[:,2,2].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    divider = make_axes_locatable(axs[0,2])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd[matid*6+3]
    except:
        print("error")
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[1, 0].set_title(r"$\epsilon_{12}$ (%)", loc='center', fontsize=8)
    im=axs[1, 0].imshow(strain_matrix_plot[:,0,1].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    axs[1, 0].set_xticks([])
    axs[1, 0].set_yticks([])
    divider = make_axes_locatable(axs[1,0])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd[matid*6+4]
    except:
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[1, 1].set_title(r"$\epsilon_{13}$ (%)", loc='center', fontsize=8)
    im=axs[1, 1].imshow(strain_matrix_plot[:,0,2].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    axs[1, 1].set_xticks([])
    divider = make_axes_locatable(axs[1,1])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd[matid*6+5]
    except:
        print("error")
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[1, 2].set_title(r"$\epsilon_{23}$ (%)", loc='center', fontsize=8)
    im = axs[1, 2].imshow(strain_matrix_plot[:,1,2].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    axs[1, 2].set_xticks([]) 
    divider = make_axes_locatable(axs[1,2])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 

    for ax in axs.flat:
        ax.label_outer()
    plt.savefig(save_directory_+"\\"+'figure_strainCRYSTAL_UB_'+str(matid)+"_"+str(index)+'.png', bbox_inches='tight',format='png', dpi=1000) 
    plt.close(fig)
    
    #%%
    fig = plt.figure(figsize=(11.69,8.27), dpi=100)
    bottom, top = 0.1, 0.9
    left, right = 0.1, 0.8
    fig.subplots_adjust(top=top, bottom=bottom, left=left, right=right, hspace=0.15, wspace=0.25)
    
    try:
        vmin, vmax = mu_sd1[matid*6]
    except:
        print("error")
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs = fig.subplots(2, 3)
    axs[0, 0].set_title(r"$\epsilon_{11}$ (%)", loc='center', fontsize=8)
    im=axs[0, 0].imshow(strain_matrix_plot[:,0,0].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    axs[0, 0].set_xticks([])
    axs[0, 0].set_yticks([])
    divider = make_axes_locatable(axs[0,0])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd1[matid*6+1]
    except:
        print("error")
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[0, 1].set_title(r"$\epsilon_{22}$ (%)", loc='center', fontsize=8)
    im=axs[0, 1].imshow(strain_matrix_plot[:,1,1].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    divider = make_axes_locatable(axs[0,1])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd1[matid*6+2]
    except:
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[0, 2].set_title(r"$\epsilon_{33}$ (%)", loc='center', fontsize=8)
    im=axs[0, 2].imshow(strain_matrix_plot[:,2,2].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    divider = make_axes_locatable(axs[0,2])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd1[matid*6+3]
    except:
        print("error")
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[1, 0].set_title(r"$\epsilon_{12}$ (%)", loc='center', fontsize=8)
    im=axs[1, 0].imshow(strain_matrix_plot[:,0,1].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    axs[1, 0].set_xticks([])
    axs[1, 0].set_yticks([])
    divider = make_axes_locatable(axs[1,0])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd1[matid*6+4]
    except:
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[1, 1].set_title(r"$\epsilon_{13}$ (%)", loc='center', fontsize=8)
    im=axs[1, 1].imshow(strain_matrix_plot[:,0,2].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    axs[1, 1].set_xticks([])
    divider = make_axes_locatable(axs[1,1])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 
    
    try:
        vmin, vmax = mu_sd1[matid*6+5]
    except:
        print("error")
        vmin = rangemin
        vmax = rangemax
    # max_vm = np.max((np.abs(vmax),np.abs(vmin)))
    # vmin = -max_vm
    # vmax = max_vm
    axs[1, 2].set_title(r"$\epsilon_{23}$ (%)", loc='center', fontsize=8)
    im = axs[1, 2].imshow(strain_matrix_plot[:,1,2].reshape((lim_x, lim_y)), origin='lower', cmap=plt.cm.jet, vmin=vmin, vmax=vmax)
    axs[1, 2].set_xticks([]) 
    divider = make_axes_locatable(axs[1,2])
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = fig.colorbar(im, cax=cax, orientation='vertical')
    cbar.ax.tick_params(labelsize=8) 

    for ax in axs.flat:
        ax.label_outer()
    plt.savefig(save_directory_+"\\"+'figure_strainCRYSTAL_UB_'+str(matid)+"_"+str(index)+'_localscale.png', bbox_inches='tight',format='png', dpi=1000) 
    plt.close(fig)
#%% write all rot mat in one ctf file for IPF plot
## write MTEX file
rotation_matrix = [[] for i in range(len(rotation_matrix1))]
for i in range(len(rotation_matrix1)):
    rotation_matrix[i].append(np.zeros((lim_x*lim_y,3,3)))

for i in range(len(rotation_matrix1)):
    temp_mat = rotation_matrix1[i][0]    
    for j in range(len(temp_mat)):
        orientation_matrix = temp_mat[j,:,:]
        ## rotate orientation by 40degrees to bring in Sample RF
        omega = np.deg2rad(-40.0)
        # # rotation de -omega autour de l'axe x (or Y?) pour repasser dans Rsample
        cw = np.cos(omega)
        sw = np.sin(omega)
        mat_from_lab_to_sample_frame = np.array([[cw, 0.0, sw], [0.0, 1.0, 0.0], [-sw, 0, cw]]) #Y
        orientation_matrix = np.dot(mat_from_lab_to_sample_frame.T, orientation_matrix)
        if np.linalg.det(orientation_matrix) < 0:
            orientation_matrix = -orientation_matrix
        rotation_matrix[i][0][j,:,:] = orientation_matrix    
# =================CALCULATION OF POSITION=====================================
for index in range(len(rotation_matrix1)):
    euler_angles = np.zeros((len(rotation_matrix[index][0]),3))
    phase_euler_angles = np.zeros(len(rotation_matrix[index][0]))
    for i in range(len(rotation_matrix[index][0])):
        if np.all(rotation_matrix[index][0][i,:,:] == 0):
            continue
        euler_angles[i,:] = rot_mat_to_euler(rotation_matrix[index][0][i,:,:])
        phase_euler_angles[i] = mat_global[index][0][i]    
        
    if index == 0:
        all_euler = euler_angles
        all_phase = phase_euler_angles
    else:
        all_euler = np.vstack((all_euler, euler_angles))
        all_phase = np.hstack((all_phase, phase_euler_angles))

lattice = lattice_
material0_LG = "3"
header = [
        "Channel Text File",
        "Prj     lauetoolsnn",
        "Author    [Ravi raj purohit]",
        "JobMode    Grid",
        "XCells    "+str(1),
        "YCells    "+str(len(all_euler)),
        "XStep    1.0",
        "YStep    1.0",
        "AcqE1    0",
        "AcqE2    0",
        "AcqE3    0",
        "Euler angles refer to Sample Coordinate system (CS0)!    Mag    100    Coverage    100    Device    0    KV    15    TiltAngle    40    TiltAxis    0",
        "Phases    1",
        str(lattice._lengths[0]*10)+";"+str(lattice._lengths[1]*10)+";"+\
        str(lattice._lengths[2]*10)+"\t"+str(lattice._angles[0])+";"+\
            str(lattice._angles[1])+";"+str(lattice._angles[2])+"\t"+"Material1"+ "\t"+material0_LG+ "\t"+"????"+"\t"+"????",
        "Phase    X    Y    Bands    Error    Euler1    Euler2    Euler3    MAD    BC    BS"]

filename125 = save_directory_+ "//"+material_+"_MTEX_ALL_UBmat.ctf"
    
f = open(filename125, "w")
for ij in range(len(header)):
    f.write(header[ij]+" \n")
    
y_step = 0
for j123 in range(all_euler.shape[0]):
    x_step = 1 * j123
    phase_id = int(all_phase[j123])
    eul =  str(phase_id)+'\t' + "%0.4f" % x_step +'\t'+"%0.4f" % y_step+'\t8\t0\t'+ \
                        "%0.4f" % all_euler[j123,0]+'\t'+"%0.4f" % all_euler[j123,1]+ \
                            '\t'+"%0.4f" % all_euler[j123,2]+'\t0.0001\t180\t0\n'
    string = eul
    f.write(string)
f.close()



# -*- coding: utf-8 -*-
import numpy as np
import scipy.optimize as sciopt

def lorentzian(height, center_x, center_y, width_x, width_y):
    """Returns a gaussian function with the given parameters"""
    width_x = float(width_x) ** 2
    width_y = float(width_y) ** 2
    return lambda x, y: height / (
        1.0 + 4 * (x - center_x) ** 2 / width_x + 4 * (y - center_y) ** 2 / width_y)


def moments(data):
    """Returns (height, x, y, width_x, width_y)
    the gaussian parameters of a 2D distribution by calculating its
    moments """
    total = data.sum()
    X, Y = np.indices(data.shape)
    x = (X * data).sum() / total
    y = (Y * data).sum() / total
    col = data[:, int(y)]
    width_x = np.sqrt(abs((np.arange(col.size) - y) ** 2 * col).sum() / col.sum())
    row = data[int(x), :]
    width_y = np.sqrt(abs((np.arange(row.size) - x) ** 2 * row).sum() / row.sum())
    height = data.max()
    return height, x, y, width_x, width_y


def fitlorentzian(data):
    """Returns (height, x, y, width_x, width_y)
    the lorentzian parameters of a 2D distribution found by a fit"""
    params = moments(data)
    errorfunction = lambda p123: np.ravel(lorentzian(*p123)(*np.indices(data.shape)) - data)
    p123, _ = sciopt.leastsq(errorfunction, params)
    return p123


def momentsr(data, circle, rotate, vheight):
    """Returns (height, amplitude, x, y, width_x, width_y, rotation angle)
    the gaussian parameters of a 2D distribution by calculating its
    moments.  Depending on the input parameters, will only output
    a subset of the above"""
    print("Not yet implemented for lorentzian")
    pass
    # total = data.sum()
    # X, Y = indices(data.shape)
    # x = (X*data).sum()/total
    # y = (Y*data).sum()/total
    # col = data[:, int(y)]
    # width_x = sqrt(abs((arange(col.size)-y)**2*col).sum()/col.sum())
    # row = data[int(x), :]
    # width_y = sqrt(abs((arange(row.size)-x)**2*row).sum()/row.sum())
    # width = ( width_x + width_y ) / 2.
    # height = stats.mode(data.ravel())[0][0]
    # amplitude = data.max()-height
    # mylist = [amplitude,x,y]
    # if vheight==1:
    # mylist = [height] + mylist
    # if circle==0:
    # mylist = mylist + [width_x,width_y]
    # else:
    # mylist = mylist + [width]
    # if rotate==1:
    # mylist = mylist + [0.] #rotation "moment" is just zero...
    # return tuple(mylist)


def twodlorentzian(inpars, circle, rotate, vheight):
    """Returns a 2d gaussian function of the form:
        x' = cos(rota) * x - sin(rota) * y
        y' = sin(rota) * x + cos(rota) * y
        (rota should be in degrees)
        g = b + a /( 1 + 4*(x-center_x)**2/width_x +  4*(y-center_y)**2/width_y  )

        where x and y are the input parameters of the returned function,
        and all other parameters are specified by this function

        However, the above values are passed by list.  The list should be:
        inpars = (height,amplitude,center_x,center_y,width_x,width_y,rota)

        You can choose to ignore / neglect some of the above input parameters using the following options:
            circle=0 - default is an elliptical gaussian (different x, y widths), but can reduce
                the input by one parameter if it's a circular gaussian
            rotate=1 - default allows rotation of the gaussian ellipse.  Can remove last parameter
                by setting rotate=0
            vheight=1 - default allows a variable height-above-zero, i.e. an additive constant
                for the Gaussian function.  Can remove first parameter by setting this to 0
        """
    inpars_old = inpars
    inpars = list(inpars)
    if vheight == 1:
        height = inpars.pop(0)
        height = float(height)
    else:
        height = float(0)
    amplitude, center_x, center_y = inpars.pop(0), inpars.pop(0), inpars.pop(0)
    amplitude = float(amplitude)
    center_x = float(center_x)
    center_y = float(center_y)
    if circle == 1:
        width = inpars.pop(0)
        width_x = float(width)
        width_y = float(width)
    else:
        width_x, width_y = inpars.pop(0), inpars.pop(0)
        width_x = float(width_x)
        width_y = float(width_y)
    if rotate == 1:
        rota = inpars.pop(0)
        rota = np.pi / 180.0 * float(rota)
        rcen_x = center_x * np.cos(rota) - center_y * np.sin(rota)
        rcen_y = center_x * np.sin(rota) + center_y * np.cos(rota)
    else:
        rcen_x = center_x
        rcen_y = center_y
    if len(inpars) > 0:
        raise ValueError("There are still input parameters:"
            + str(inpars) + " and you've input: "
            + str(inpars_old) + " circle=%d, rotate=%d, vheight=%d" % (circle, rotate, vheight))

    def rotlorentz(x, y):
        """
        a*w**2/((x-x0)**2+w**2)

        """
        if rotate == 1:
            xp = x * np.cos(rota) - y * np.sin(rota)
            yp = x * np.sin(rota) + y * np.cos(rota)
        else:
            xp = x
            yp = y
        # g = height+amplitude/( 1 + 4*(x-rcen_x)**2/width_x +  4*(y-rcen_y)**2/width_y  )
        g = height + amplitude / (1.0 + 4 * (xp - rcen_x) ** 2 / width_x) / (
            1 + 4 * (yp - rcen_y) ** 2 / width_y)
        return g

    return rotlorentz


def lorentzfit(data, err=None, params=[], autoderiv=1, return_all=0,
                                                        circle=0,
                                                        rotate=1,
                                                        vheight=1,
                                                        xtol=0.0000001):
    """
    Lorentzian fitter with the ability to fit a variety of different forms of 2-dimensional gaussian.

    Input Parameters:
        data - 2-dimensional data array
        err=None - error array with same size as data array
        params=[] - initial input parameters for Gaussian function.
            (height, amplitude, x, y, width_x, width_y, rota)
            if not input, these will be determined from the moments of the system,
            assuming no rotation
        autoderiv=1 - use the autoderiv provided in the lmder.f function (the alternative
            is to us an analytic derivative with lmdif.f: this method is less robust)
        return_all=0 - Default is to return only the Gaussian parameters.  See below for
            detail on output
        circle=0 - default is an elliptical gaussian (different x, y widths), but can reduce
            the input by one parameter if it's a circular gaussian
        rotate=1 - default allows rotation of the gaussian ellipse.  Can remove last parameter
            by setting rotate=0
        vheight=1 - default allows a variable height-above-zero, i.e. an additive constant
            for the Gaussian function.  Can remove first parameter by setting this to 0

    Output:
        Default output is a set of Gaussian parameters with the same shape as the input parameters
        Can also output the covariance matrix, 'infodict' that contains a lot more detail about
            the fit (see scipy.optimize.leastsq), and a message from leastsq telling what the exit
            status of the fitting routine was

        Warning: Does NOT necessarily output a rotation angle between 0 and 360 degrees.
    """
    if params == []:
        params = momentsr(data, circle, rotate, vheight)
    if err is None:
        errorfunction = lambda p123: np.ravel(
            (twodlorentzian(p123, circle, rotate, vheight)(*np.indices(data.shape)) - data))
    else:
        errorfunction = lambda p123: np.ravel(
            (twodlorentzian(p123, circle, rotate, vheight)(*np.indices(data.shape)) - data) / err)
    if autoderiv == 0:
        # the analytic derivative, while not terribly difficult, is less efficient and useful.  I only bothered
        # putting it here because I was instructed to do so for a class project - please ask if you would like
        # this feature implemented
        raise ValueError("I'm sorry, I haven't implemented this feature yet.")
    else:
        p123, cov, infodict, errmsg, _ = sciopt.leastsq(
            errorfunction, params, full_output=1, xtol=xtol)
    if return_all == 0:
        return p123
    elif return_all == 1:
        return p123, cov, infodict, errmsg

# if __name__ == "__main__":

#     if 0:
#         # Create the gaussian data
#         # -----------------------------------------------------
#         Xin, Yin = np.mgrid[0:201, 0:201]
#         # data = gaussian(3, 100, 100, 20, 40)(Xin, Yin) + random.random(Xin.shape)
#         # inpars = (height,amplitude,center_x,center_y,width_x,width_y,rota)
#         inpars = (0, 200, 50, 50, 10, 40, 45)
#         inparsshort = inpars[:5]
#         circle = 0
#         rotate = 1
#         vheight = 1
#         data = twodlorentzian(inpars, circle, rotate, vheight)(Xin, Yin) + 10 * np.random.random(Xin.shape)
#         # -------------------------------------

#     fifi = "Zr_A169_0220.mccd"
#     fifi = "CKRMONO.0057"
#     fifi = "Ge_run41_1_0003.mccd"

#     center_pixel = (1296, 2048 - 1021)  # XMAS and array convention (x,2048-yfit2d)
#     center_pixel = (1079, 725)
#     center_pixel = (1588, 818)
#     xboxsize, yboxsize = 20, 20

#     dat = IOimage.readoneimage_crop(fifi, center_pixel, (xboxsize, yboxsize))

#     def fromindex_to_pixelpos_x(index, _):
#         return center_pixel[0] - xboxsize + index

#     def fromindex_to_pixelpos_y(index, _):
#         return center_pixel[1] - yboxsize + index

#     # ax = axes()
#     # ax.xaxis.set_major_formatter(FuncFormatter(fromindex_to_pixelpos_x))
#     # ax.yaxis.set_major_formatter(FuncFormatter(fromindex_to_pixelpos_y))
#     # imshow(dat,interpolation='nearest')#,origin='lower')
#     # show()

#     start_baseline = np.amin(dat)
#     # start_j,start_i=yboxsize,xboxsize # from input center
#     start_j, start_i = (np.argmax(dat) // dat.shape[1], np.argmax(dat) % dat.shape[1])  # from maximum intensity in dat
#     # start_amplitude=amax(dat)-start_baseline
#     start_amplitude = dat[start_j, start_i] - start_baseline
#     start_sigma1, start_sigma2 = 10, 5
#     start_anglerot = 0
#     startingparams = [start_baseline,
#                         start_amplitude,
#                         start_j,
#                         start_i,
#                         start_sigma1,
#                         start_sigma2,
#                         start_anglerot]

#     print("startingparams")
#     print(startingparams)

#     if all(dat > 0):
#         print("logscale")
#         p.matshow(np.log(dat), cmap=GT.GIST_EARTH_R, interpolation="nearest", origin="upper")
#     else:
#         p.imshow(dat, cmap=GT.GIST_EARTH_R, interpolation="nearest", origin="upper")

#     params, cov, infodict, errmsg = lorentzfit(dat,
#                                             err=None,
#                                             params=startingparams,
#                                             autoderiv=1,
#                                             return_all=1,
#                                             circle=0,
#                                             rotate=1,
#                                             vheight=1)

#     print("\n *****fitting results with Lorentzian************\n")
#     print(params)
#     print("background intensity:            %.2f" % params[0])
#     print("Peak amplitude above background        %.2f" % params[1])
#     print("pixel position (X)            %.2f" % (params[3] - xboxsize + center_pixel[0]))
#     print("pixel position (Y)            %.2f" % (params[2] - yboxsize + center_pixel[1]))
#     print("std 1,std 2 (pix)            ( %.2f , %.2f )" % (params[4], params[5]))
#     print("e=min(std1,std2)/max(std1,std2)        %.3f"
#         % (min(params[4], params[5]) / max(params[4], params[5])))
#     print("Rotation angle (deg)            %.2f" % (params[6] % 360))
#     print("************************************\n")
#     print(params)
#     inpars_res = params
#     fit = twodlorentzian(inpars_res, 0, 1, 1)

#     # params = fitgaussian(data)
#     # fit = gaussian(*params)

#     # params1=[  8.37532076e+01 ,  7.46353362e+01 ,  2.30777891e+01+11 ,  2.23761344e+01+8,
#     # 6.05357888e+00  , 1.20693476e+01 , -1.16509603e+06]
#     # params2=[ 79.3635639 ,  93.12639397  , 9.3952523 +70,  29.51984472+22 ,  5.46769477
#     # ,  17.71425257 , 84.66087334]
#     # params3=[ 89.21628068  ,53.17974564,  9.98964225+18 ,  5.45738096+59 , 11.84612735,
#     # 5.08762344 , 30.90213561]
#     # fit=twodlorentzian(params,0,1,1)
#     # fit1=twodlorentzian(params1,0,1,1)
#     # fit2=twodlorentzian(params2,0,1,1)
#     # fit3=twodlorentzian(params3,0,1,1)

#     isointensity = np.linspace(1000, np.amax(dat), 20)
#     p.contour(fit(*np.indices(dat.shape)), isointensity, cmap=GT.COPPER)
#     # contour(fit(*indices(dat.shape))+fit1(*indices(dat.shape))+fit2(*indices(dat.shape))+fit3(*indices(dat.shape)), isointensity,cmap=cm.copper)
#     ax = p.gca()
#     ax.xaxis.set_major_formatter(p.FuncFormatter(fromindex_to_pixelpos_x))
#     ax.yaxis.set_major_formatter(p.FuncFormatter(fromindex_to_pixelpos_y))
#     # (height, x, y, width_x, width_y) = params

#     # text(0.95, 0.05, """
#     # x : %.1f
#     # y : %.1f
#     # width_x : %.1f
#     # width_y : %.1f""" %(x, y, width_x, width_y),
#     # fontsize=16, horizontalalignment='right',
#     # verticalalignment='bottom', transform=ax.transAxes)

#     p.show()



import os
import setuptools
from setuptools import find_packages, setup
#["lauetoolsnn"],
with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    long_description = readme.read()

setuptools.setup(
    name="lauetoolsnn",
    version="3.0.22",
    author="Ravi raj purohit PURUSHOTTAM RAJ PUROHIT",
    author_email="purushot@esrf.fr",
    description="GUI routine for Laue neural network training and prediction- v3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    packages=find_packages(),
    url="https://github.com/ravipurohit1991/lauetoolsnn",
    install_requires=['matplotlib', 'Keras', 'fast_histogram', 'numpy', 'h5py', 'tensorflow', 'PyQt5', 'scikit_learn', 'fabio', 'networkx'],
    entry_points={
                 "console_scripts": ["lauetoolsnn=lauetoolsnn.lauetoolsneuralnetwork:start"]
                 },
    classifiers=[
                "Programming Language :: Python :: 3",
                "License :: OSI Approved :: MIT License",
                "Operating System :: OS Independent",
                ],
    python_requires='>=3.6',
)


# -*- coding: utf-8 -*-
"""
module to fit 2d value in array

still in development for fit of multiple peaks in ROI
"""
import pylab as p
import numpy as np

from scipy import optimize, stats

# from matplotlib.ticker import FuncFormatter

try:
    from lauetools import generaltools as GT 
except:
    import lauetoolsnn.lauetools.generaltools as GT

USE_CYTHON = False

def gaussian(height, center_x, center_y, width_x, width_y):
    """Returns a gaussian function with the given parameters"""
    width_x = float(width_x)
    width_y = float(width_y)
    return lambda x, y: height * np.exp(
        -(((center_x - x) / width_x) ** 2 + ((center_y - y) / width_y) ** 2) / 2)


def moments(data):
    """Returns (height, x, y, width_x, width_y)
    the gaussian parameters of a 2D distribution by calculating its
    moments """
    total = data.sum() * 1.0
    X, Y = np.indices(data.shape)
    x = (X * data).sum() / total
    y = (Y * data).sum() / total
    col = data[:, int(y)] * 1.0
    width_x = np.sqrt(np.abs((np.arange(col.size) - y) ** 2 * col).sum() / col.sum())
    row = data[int(x), :] * 1.0
    width_y = np.sqrt(np.abs((np.arange(row.size) - x) ** 2 * row).sum() / row.sum())
    height = data.max()
    return height, x, y, width_x, width_y


def fitgaussian(data):
    """Returns (height, x, y, width_x, width_y)
    the gaussian parameters of a 2D distribution found by a fit"""
    params = moments(data)
    errorfunction = lambda p: np.ravel(gaussian(*p)(*np.indices(data.shape)) - data)
    p, _ = optimize.leastsq(errorfunction, params)
    return p


def momentsr(data, circle, rotate, vheight):
    """Returns (height, amplitude, x, y, width_x, width_y, rotation angle)
    the gaussian parameters of a 2D distribution by calculating its
    moments.  Depending on the input parameters, will only output
    a subset of the above"""
    total = data.sum() * 1.0
    X, Y = np.indices(data.shape)
    x = (X * data).sum() / total
    y = (Y * data).sum() / total
    col = data[:, int(y)] * 1.0
    width_x = np.sqrt(np.abs((np.arange(col.size) - y) ** 2 * col).sum() / col.sum())
    row = data[int(x), :] * 1.0
    width_y = np.sqrt(np.abs((np.arange(row.size) - x) ** 2 * row).sum() / row.sum())
    width = (width_x + width_y) / 2.0
    height = stats.mode(data.ravel())[0][0]
    amplitude = data.max() - height
    mylist = [amplitude, x, y]
    if vheight == 1:
        mylist = [height] + mylist
    if circle == 0:
        mylist = mylist + [width_x, width_y]
    else:
        mylist = mylist + [width]
    if rotate == 1:
        mylist = mylist + [0.0]  # rotation "moment" is just zero...
    return tuple(mylist)


def twodgaussian(inpars, circle, rotate, vheight):
    """Returns a 2d gaussian function of the form:
        x' = cos(rota) * x - sin(rota) * y
        y' = sin(rota) * x + cos(rota) * y
        (rota should be in degrees)
        g = b + a exp ( - ( ((x-center_x)/width_x)**2 +
        ((y-center_y)/width_y)**2 ) / 2 )

        where x and y are the input parameters of the returned function,
        and all other parameters are specified by this function

        However, the above values are passed by list.  The list should be:
        inpars = (height,amplitude,center_x,center_y,width_x,width_y,rota)

        You can choose to ignore / neglect some of the above input parameters using the following options:
            circle=0 - default is an elliptical gaussian (different x, y widths), but can reduce
                the input by one parameter if it's a circular gaussian
            rotate=1 - default allows rotation of the gaussian ellipse.  Can remove last parameter
                by setting rotate=0
            vheight=1 - default allows a variable height-above-zero, i.e. an additive constant
                for the Gaussian function.  Can remove first parameter by setting this to 0
        """
    inpars_old = inpars
    inpars = list(inpars)
    if vheight == 1:
        height = inpars.pop(0)
        height = float(height)
    else:
        height = float(0)
    amplitude, center_x, center_y = inpars.pop(0), inpars.pop(0), inpars.pop(0)
    amplitude = float(amplitude)
    center_x = float(center_x)
    center_y = float(center_y)
    if circle == 1:
        width = inpars.pop(0)
        width_x = float(width)
        width_y = float(width)
    else:
        width_x, width_y = inpars.pop(0), inpars.pop(0)
        width_x = float(width_x)
        width_y = float(width_y)
    if rotate == 1:
        rota = inpars.pop(0)
        rota = np.pi / 180.0 * float(rota)
        rcen_x = center_x * np.cos(rota) - center_y * np.sin(rota)
        rcen_y = center_x * np.sin(rota) + center_y * np.cos(rota)
    else:
        rcen_x = center_x
        rcen_y = center_y
    if len(inpars) > 0:
        raise ValueError(
            "There are still input parameters:"
            + str(inpars)
            + " and you've input: "
            + str(inpars_old)
            + " circle=%d, rotate=%d, vheight=%d" % (circle, rotate, vheight))

    def rotgauss(x, y):
        if rotate == 1:
            xp = x * np.cos(rota) - y * np.sin(rota)
            yp = x * np.sin(rota) + y * np.cos(rota)
        else:
            xp = x
            yp = y
        g = height + amplitude * np.exp(
            -(((rcen_x - xp) / width_x) ** 2 + ((rcen_y - yp) / width_y) ** 2) / 2.0)
        return g

    return rotgauss

def twodgaussian_2peaks(inpars, circle, rotate, vheight):
    """Returns a 2d gaussian function of the form:
    x' = cos(rota) * x - sin(rota) * y
    y' = sin(rota) * x + cos(rota) * y
    (rota should be in degrees)
    g = b + a exp ( - ( ((x-center_x)/width_x)**2 +
    ((y-center_y)/width_y)**2 ) / 2 )

    where x and y are the input parameters of the returned function,
    and all other parameters are specified by this function

    However, the above values are passed by list.  The list should be:
    inpars = (height,amplitude,center_x,center_y,width_x,width_y,rota, # for gaussian 1
                amplitude,center_x,center_y,width_x,width_y,rota)) for gaussian 2

    You can choose to ignore / neglect some of the above input parameters using the following options:
        circle=0 - default is an elliptical gaussian (different x, y widths), but can reduce
            the input by one parameter if it's a circular gaussian
        rotate=1 - default allows rotation of the gaussian ellipse.  Can remove last parameter
            by setting rotate=0
        vheight=1 - default allows a variable height-above-zero, i.e. an additive constant
            for the Gaussian function.  Can remove first parameter by setting this to 0
    """
    # inpars_old = inpars
    inpars = list(inpars)

    if vheight == 1:
        # height = inpars.pop(0)
        height_1 = float(inpars[0])

    else:
        print("Still Not implemented in twodgaussian_2peaks()")
        # height_1 = float(0)
        # height_2 = float(0)

    amplitude_1, center_x_1, center_y_1 = inpars[1:4]
    amplitude_2, center_x_2, center_y_2 = inpars[7:10]

    amplitude_1 = float(amplitude_1)
    amplitude_2 = float(amplitude_2)
    center_x_1 = float(center_x_1)
    center_y_1 = float(center_y_1)
    center_x_2 = float(center_x_2)
    center_y_2 = float(center_y_2)

    if circle == 1:
        print("Still Not implemented in twodgaussian_2peaks()")
        # width = inpars.pop(0)
        # width_x = float(width)
        # width_y = float(width)
    else:
        width_x_1, width_y_1 = inpars[4:6]
        width_x_1 = float(width_x_1)
        width_y_1 = float(width_y_1)
        width_x_2, width_y_2 = inpars[10:12]
        width_x_2 = float(width_x_2)
        width_y_2 = float(width_y_2)
    if rotate == 1:
        rota_1 = inpars[6]
        rota_1 = np.pi / 180.0 * float(rota_1)
        rcen_x_1 = center_x_1 * np.cos(rota_1) - center_y_1 * np.sin(rota_1)
        rcen_y_1 = center_x_1 * np.sin(rota_1) + center_y_1 * np.cos(rota_1)
        rota_2 = inpars[12]
        rota_2 = np.pi / 180.0 * float(rota_2)
        rcen_x_2 = center_x_2 * np.cos(rota_2) - center_y_2 * np.sin(rota_2)
        rcen_y_2 = center_x_2 * np.sin(rota_2) + center_y_2 * np.cos(rota_2)
    else:
        print("Still Not implemented in twodgaussian_2peaks()")
        # rcen_x = center_x
        # rcen_y = center_y
    # if len(inpars) > 0:
    # raise ValueError("There are still input parameters:" + str(inpars) + \
    # " and you've input: " + str(inpars_old) + " circle=%d, rotate=%d, vheight=%d" % (circle,rotate,vheight) )

    # def rotgauss_1(x,y):
    # if rotate==1:
    # xp = x * cos(rota_1) - y * sin(rota_1)
    # yp = x * sin(rota_1) + y * cos(rota_1)
    # else:
    # xp = x
    # yp = y
    # g1 = height_1+amplitude_1*exp(
    # -(((rcen_x_1-xp)/width_x_1)**2+
    # ((rcen_y_1-yp)/width_y_1)**2)/2.)
    # return g1

    # def rotgauss_2(x,y):
    # if rotate==1:
    # xp = x * cos(rota_2) - y * sin(rota_2)
    # yp = x * sin(rota_2) + y * cos(rota_2)
    # else:
    # xp = x
    # yp = y
    # g2 = height_2+amplitude_2*exp(
    # -(((rcen_x_2-xp)/width_x_2)**2+
    # ((rcen_y_2-yp)/width_y_2)**2)/2.)
    # return g2

    def rotgauss_2peaks(x, y):

        xp_1 = x * np.cos(rota_1) - y * np.sin(rota_1)
        yp_1 = x * np.sin(rota_1) + y * np.cos(rota_1)
        xp_2 = x * np.cos(rota_2) - y * np.sin(rota_2)
        yp_2 = x * np.sin(rota_2) + y * np.cos(rota_2)

        g = (height_1 + amplitude_1
            * np.exp(-(((rcen_x_1 - xp_1) / width_x_1) ** 2
                    + ((rcen_y_1 - yp_1) / width_y_1) ** 2)/ 2.0)
            + amplitude_2
            * np.exp(-(((rcen_x_2 - xp_2) / width_x_2) ** 2
                    + ((rcen_y_2 - yp_2) / width_y_2) ** 2)/ 2.0))

        return g

    return rotgauss_2peaks

    # if flag ==1:
    # return rotgauss_1
    # if flag ==2:
    # return rotgauss_2


def gaussfit( data, err=None, params=[], autoderiv=1, return_all=0, circle=0,
    rotate=1, vheight=1, xtol=0.0000001, Acceptable_HighestValue=False,
    Acceptable_LowestValue=False, ijindices_array=None,
    usecythonmodule=USE_CYTHON, computerrorbars=False):
    """
    Gaussian fitter with the ability to fit a variety of different forms of 2-dimensional gaussian.

    Input Parameters:
        data - 2-dimensional data array
        err=None - error array with same size as data array
        params=[] - initial input parameters for Gaussian function.
            (height, amplitude, x, y, width_x, width_y, rota)
            if not input, these will be determined from the moments of the system,
            assuming no rotation
        autoderiv=1 - use the autoderiv provided in the lmder.f function (the alternative
            is to us an analytic derivative with lmdif.f: this method is less robust)
        return_all=0 - Default is to return only the Gaussian parameters.  See below for
            detail on output
        circle=0 - default is an elliptical gaussian (different x, y widths), but can reduce
            the input by one parameter if it's a circular gaussian
        rotate=1 - default allows rotation of the gaussian ellipse.  Can remove last parameter
            by setting rotate=0
        vheight=1 - default allows a variable height-above-zero, i.e. an additive constant
            for the Gaussian function.  Can remove first parameter by setting this to 0

        Acceptable_HighestValue: if not False, pixel intensity level above which the pixel has a zero weight
        Acceptable_LowestValue: if not False, pixel intensity level below which the pixel has a zero weight

    Output:
        Default output is a set of Gaussian parameters with the same shape as the input parameters
        Can also output the covariance matrix, 'infodict' that contains a lot more detail about
            the fit (see scipy.optimize.leastsq), and a message from leastsq telling what the exit
            status of the fitting routine was

        Warning: Does NOT necessarily output a rotation angle between 0 and 360 degrees.
    """
    #     data = np.ma.array(data, mask=False)
    #
    #     data.mask[data >= 65535] = True
    # print("in gaussfit")
    if (Acceptable_HighestValue is not False) or (Acceptable_LowestValue is not False):
        err = np.ones(data.shape)
    if Acceptable_HighestValue is not False:
        err[data >= Acceptable_HighestValue] = 0
    if Acceptable_LowestValue is not False:
        err[data <= Acceptable_LowestValue] = 0

    if params == []:
        params = momentsr(data, circle, rotate, vheight)

    if ijindices_array is None:
        ijindices_array = np.indices(data.shape)

    if not usecythonmodule:
        if err is None:
            errorfunction = lambda p: np.ravel(
                (twodgaussian(p, circle, rotate, vheight)(*ijindices_array) - data))
        else:
            errorfunction = lambda p: np.ravel(
                (twodgaussian(p, circle, rotate, vheight)(*ijindices_array) - data)
                * err)
            
    if autoderiv == 0:
        # the analytic derivative, while not terribly difficult, is less efficient and useful.
        # I only bothered putting it here because I was instructed to do so for a class project.
        #  - please ask if you would like    # this feature implemented
        raise ValueError("I'm sorry, I haven't implemented this feature yet.")
    else:
        (p, cov, infodict, errmsg, success) = optimize.leastsq(
            errorfunction, params, full_output=1, xtol=xtol)

        if computerrorbars:
            # https://stackoverflow.com/questions/14581358/getting-standard-errors-on-fitted-parameters-using-the-optimize-leastsq-method-i/21844726
 
            n1, n2 = data.shape

            if (n1*n2 > len(p)) and cov is not None:
                s_sq = (errorfunction(p)**2).sum()/(n1*n2-len(p))
                pcov = cov * s_sq
            else:
                pcov = np.inf

            error = []
            for i in range(len(p)):
                try:
                    error.append(np.absolute(pcov[i][i])**0.5)
                except:
                    error.append(0.00)
            pfit_leastsq = p
            perr_leastsq = np.array(error)

            print("pfit_leastsq", pfit_leastsq)
            print("perr_leastsq", perr_leastsq)

    if return_all == 0:
        return p
    elif return_all == 1:
        return p, cov, infodict, errmsg


def gaussfit_2peaks( data, err=None, params=[], autoderiv=1, return_all=0,
                                                circle=0, rotate=1, vheight=1, xtol=0.0000001):
    """
    two Gaussians fitter with the ability to fit a variety of
    different forms of 2-dimensional gaussian.

    Input Parameters:
        data - 2-dimensional data array
        err=None - error array with same size as data array
        params=[] - initial input parameters for Gaussian function.
            (height, amplitude, x, y, width_x, width_y, rota)
            if not input, these will be determined from the moments of the system,
            assuming no rotation
        autoderiv=1 - use the autoderiv provided in the lmder.f function (the alternative
            is to us an analytic derivative with lmdif.f: this method is less robust)
        return_all=0 - Default is to return only the Gaussian parameters.  See below for
            detail on output
        circle=0 - default is an elliptical gaussian (different x, y widths), but can reduce
            the input by one parameter if it's a circular gaussian
        rotate=1 - default allows rotation of the gaussian ellipse.  Can remove last parameter
            by setting rotate=0
        vheight=1 - default allows a variable height-above-zero, i.e. an additive constant
            for the Gaussian function.  Can remove first parameter by setting this to 0

    Output:
        Default output is a set of Gaussian parameters with the same shape as the input parameters
        Can also output the covariance matrix, 'infodict' that contains a lot more detail about
            the fit (see scipy.optimize.leastsq), and a message from leastsq telling what the exit
            status of the fitting routine was

        Warning: Does NOT necessarily output a rotation angle between 0 and 360 degrees.
    """
    if params == []:
        print("I'm sorry, I haven't implemented this feature yet. in gaussfit_2peaks()")
        # params = (momentsr(data,circle,rotate,vheight))
    if err is None:
        errorfunction = lambda p: np.ravel(
            (twodgaussian_2peaks(p, circle, rotate, vheight)(*np.indices(data.shape))
                - data))
    else:
        errorfunction = lambda p: np.ravel(
            (twodgaussian(p, circle, rotate, vheight)(*np.indices(data.shape)) - data)
            / err)
    if autoderiv == 0:
        raise ValueError("I'm sorry, I haven't implemented this feature yet.")
    else:
        p, cov, infodict, errmsg, success = optimize.leastsq(
            errorfunction, params, full_output=1, xtol=xtol)
    if return_all == 0:
        return p
    elif return_all == 1:
        return p, cov, infodict, errmsg


def create2Dgaussiandata():
    # Create the gaussian data
    Xin, Yin = np.mgrid[0:201, 0:201]
    data = gaussian(3, 100, 100, 20, 40)(Xin, Yin) + np.random.random(Xin.shape)

    p.matshow(data, cmap=GT.GIST_EARTH_R)

    params = fitgaussian(data)
    fit = gaussian(*params)

    p.contour(fit(*np.indices(data.shape)), cmap=GT.COPPER)
    ax = p.gca()
    (height, x, y, width_x, width_y) = params

    p.text(0.95, 0.05,
        """
    x : %.1f
    y : %.1f
    width_x : %.1f
    width_y : %.1f"""
        % (x, y, width_x, width_y), fontsize=16, horizontalalignment="right",
        verticalalignment="bottom", transform=ax.transAxes)

    p.show()


# if __name__ == "__main__":

#     if 0:
#         # Create the gaussian data
#         # -----------------------------------------------------
#         Xin, Yin = np.mgrid[0:201, 0:201]
#         # data = gaussian(3, 100, 100, 20, 40)(Xin, Yin) + random.random(Xin.shape)
#         # inpars = (height,amplitude,center_x,center_y,width_x,width_y,rota)
#         inpars = (25, 100, 100, 100, 10, 20, 12)
#         inparsshort = inpars[:5]
#         circle = 0
#         rotate = 1
#         vheight = 1
#         data = twodgaussian(inpars, circle, rotate, vheight)(Xin, Yin) + 10 * np.random.random(Xin.shape)
#         # -------------------------------------

#     # fifi='Zr_A169_0220.mccd'
#     # fifi='CKRMONO.0057'
#     # fifi='BKr_0252.mccd'

#     fifi = "UO2_A163_2_0028.mccd"
#     dirname = "/home/micha/lauetools/trunk/Examples/UO2"

#     fifi = "He60keV_telque_mono_0007.mccd"
#     dirname = "/home/micha/LT4/"

#     CenCam = np.array([830, 1190])

#     if 0:
#         peak_1 = [778, 782]
#         peak_2 = [777, 778]
#         start_amplitude_1 = 22000
#         start_amplitude_2 = 1800
#     if 0:
#         peak_1 = np.array([609, 933])
#         peak_2 = peak_1 + (peak_1 - CenCam) * 0.01
#         # start_amplitude_1 = 22000
#         # start_amplitude_2 =1800

#     if 0:
#         peak_1 = np.array([1059, 937])
#         peak_2 = peak_1 + (peak_1 - CenCam) * 0.005
#         # start_amplitude_1 = 22000
#         # start_amplitude_2 =1800
#     if 1:
#         peak_1 = np.array([1392, 739])
#         peak_2 = peak_1 + (peak_1 - CenCam) * 0.005
#         peak_2 = [1398, 735]
#         # start_amplitude_1 = 22000
#         # start_amplitude_2 =1800

#     xboxsize, yboxsize = 20, 20

#     center_pixel = peak_1

#     if sys.version_info.major == 3:
#         from . import readmccd as RMCCD
#     else:
#         import readmccd as RMCCD

#     dat = IOimage.readoneimage_crop(
#         fifi, center_pixel, (xboxsize, yboxsize), dirname=dirname)

#     def fromindex_to_pixelpos_x(index, pos):
#         return center_pixel[0] - xboxsize + index

#     def fromindex_to_pixelpos_y(index, pos):
#         return center_pixel[1] - yboxsize + index

#     # ax = axes()
#     # ax.xaxis.set_major_formatter(FuncFormatter(fromindex_to_pixelpos_x))
#     # ax.yaxis.set_major_formatter(FuncFormatter(fromindex_to_pixelpos_y))
#     # imshow(dat,interpolation='nearest')#,origin='lower')
#     # show()

#     start_baseline = np.amin(dat)
#     # start_j,start_i=yboxsize,xboxsize # from input center
#     start_j, start_i = (
#         np.argmax(dat) // dat.shape[1],
#         np.argmax(dat) % dat.shape[1],
#     )  # from maximum intensity in dat
#     start_amplitude_1 = np.amax(dat) - start_baseline
#     start_amplitude_2 = start_amplitude_1 / 10.0
#     # start_amplitude=dat[start_j,start_i]-start_baseline
#     start_sigma1, start_sigma2 = 2, 2
#     start_anglerot = 0
#     # startingparams=[start_baseline,start_amplitude,start_j,start_i,start_sigma1,start_sigma2,start_anglerot,
#     # start_baseline,start_amplitude,start_j,start_i,start_sigma1,start_sigma2,start_anglerot]

#     startingparams = [start_baseline,
#         start_amplitude_1,
#         peak_1[1] - (center_pixel[1] - yboxsize),
#         peak_1[0] - (center_pixel[0] - xboxsize),
#         start_sigma1,
#         start_sigma2,
#         start_anglerot,
#         start_amplitude_2,
#         peak_2[1] - (center_pixel[1] - yboxsize),
#         peak_2[0] - (center_pixel[0] - xboxsize),
#         start_sigma1,
#         start_sigma2,
#         start_anglerot]

#     print("startingparams")
#     print(startingparams[:7])
#     print(startingparams[7:])

#     if 0:  # all(dat>0):
#         print("logscale")  # matshow seems slow !
#         p.matshow(
#             np.log(dat), cmap=GT.GIST_EARTH_R, interpolation="nearest", origin="upper")
#     else:
#         p.imshow(np.log(dat + 0.000000000000001), cmap=GT.GIST_EARTH_R,
#             interpolation="nearest",
#             origin="upper")

#     # fit one peak
#     params, cov, infodict, errmsg = gaussfit_2peaks(dat, err=None,
#         params=startingparams,
#         autoderiv=1,
#         return_all=1,
#         circle=0,
#         rotate=1,
#         vheight=1)

#     print("\n *****fitting results ************\n")
#     print("background intensity:            %.2f" % params[0])
#     print("Peak amplitude above background        %.2f" % params[1])
#     print("pixel position (X)            %.2f" % (params[3] - xboxsize + center_pixel[0]))
#     print("pixel position (Y)            %.2f" % (params[2] - yboxsize + center_pixel[1]))
#     print("std 1,std 2 (pix)            ( %.2f , %.2f )" % (params[4], params[5]))
#     print("e=min(std1,std2)/max(std1,std2)        %.3f"
#         % (min(params[4], params[5]) / max(params[4], params[5])))
#     print("Rotation angle (deg)            %.2f\n\n" % (params[6] % 360))

#     print("background intensity:            %.2f" % params[0])
#     print("Peak amplitude above background        %.2f" % params[7])
#     print("pixel position (X)            %.2f" % (params[9] - xboxsize + center_pixel[0]))
#     print("pixel position (Y)            %.2f" % (params[8] - yboxsize + center_pixel[1]))
#     print("std 1,std 2 (pix)            ( %.2f , %.2f )" % (params[10], params[11]))
#     print("e=min(std1,std2)/max(std1,std2)        %.3f"
#         % (min(params[10], params[11]) / max(params[10], params[11])))
#     print("Rotation angle (deg)            %.2f" % (params[12] % 360))
#     print("************************************\n")
#     # print params
#     inpars_res = params
#     fit_1 = twodgaussian_2peaks(inpars_res, 0, 1, 1)

#     # params = fitgaussian(data)
#     # fit = gaussian(*params)

#     # params1=[  8.37532076e+01 ,  7.46353362e+01 ,  2.30777891e+01+11 ,  2.23761344e+01+8,
#     # 6.05357888e+00  , 1.20693476e+01 , -1.16509603e+06]
#     # params2=[ 79.3635639 ,  93.12639397  , 9.3952523 +70,  29.51984472+22 ,  5.46769477
#     # ,  17.71425257 , 84.66087334]
#     # params3=[ 89.21628068  ,53.17974564,  9.98964225+18 ,  5.45738096+59 , 11.84612735,
#     # 5.08762344 , 30.90213561]
#     # fit=twodgaussian(params,0,1,1)
#     # fit1=twodgaussian(params1,0,1,1)
#     # fit2=twodgaussian(params2,0,1,1)
#     # fit3=twodgaussian(params3,0,1,1)
#     isointensity = np.logspace(np.log(params[0] * 0.5), np.log(max(params[1], params[7])), 40)
#     p.contour(fit_1(*np.indices(dat.shape)), isointensity, cmap=GT.COPPER)
#     # contour(fit(*indices(dat.shape))+fit1(*indices(dat.shape))+fit2(*indices(dat.shape))+fit3(*indices(dat.shape)), isointensity,cmap=cm.copper)
#     ax = p.gca()
#     ax.xaxis.set_major_formatter(FuncFormatter(fromindex_to_pixelpos_x))
#     ax.yaxis.set_major_formatter(FuncFormatter(fromindex_to_pixelpos_y))
#     # (height, x, y, width_x, width_y) = params

#     # text(0.95, 0.05, """
#     # x : %.1f
#     # y : %.1f
#     # width_x : %.1f
#     # width_y : %.1f""" %(x, y, width_x, width_y),
#     # fontsize=16, horizontalalignment='right',
#     # verticalalignment='bottom', transform=ax.transAxes)

#     p.show()
